from enum import Enum
import math

from ..abstract_unit import AbstractMeasure



class VolumetricHeatCapacityUnits(Enum):
        """
            VolumetricHeatCapacityUnits enumeration
        """
        
        JoulePerCubicMeterKelvin = 'JoulePerCubicMeterKelvin'
        """
            
        """
        
        JoulePerCubicMeterDegreeCelsius = 'JoulePerCubicMeterDegreeCelsius'
        """
            
        """
        
        CaloriePerCubicCentimeterDegreeCelsius = 'CaloriePerCubicCentimeterDegreeCelsius'
        """
            
        """
        
        BtuPerCubicFootDegreeFahrenheit = 'BtuPerCubicFootDegreeFahrenheit'
        """
            
        """
        
        KilojoulePerCubicMeterKelvin = 'KilojoulePerCubicMeterKelvin'
        """
            
        """
        
        MegajoulePerCubicMeterKelvin = 'MegajoulePerCubicMeterKelvin'
        """
            
        """
        
        KilojoulePerCubicMeterDegreeCelsius = 'KilojoulePerCubicMeterDegreeCelsius'
        """
            
        """
        
        MegajoulePerCubicMeterDegreeCelsius = 'MegajoulePerCubicMeterDegreeCelsius'
        """
            
        """
        
        KilocaloriePerCubicCentimeterDegreeCelsius = 'KilocaloriePerCubicCentimeterDegreeCelsius'
        """
            
        """
        

class VolumetricHeatCapacityDto:
    """
    A DTO representation of a VolumetricHeatCapacity

    Attributes:
        value (float): The value of the VolumetricHeatCapacity.
        unit (VolumetricHeatCapacityUnits): The specific unit that the VolumetricHeatCapacity value is representing.
    """

    def __init__(self, value: float, unit: VolumetricHeatCapacityUnits):
        """
        Create a new DTO representation of a VolumetricHeatCapacity

        Parameters:
            value (float): The value of the VolumetricHeatCapacity.
            unit (VolumetricHeatCapacityUnits): The specific unit that the VolumetricHeatCapacity value is representing.
        """
        self.value: float = value
        """
        The value of the VolumetricHeatCapacity
        """
        self.unit: VolumetricHeatCapacityUnits = unit
        """
        The specific unit that the VolumetricHeatCapacity value is representing
        """

    def to_json(self):
        """
        Get a VolumetricHeatCapacity DTO JSON object representing the current unit.

        :return: JSON object represents VolumetricHeatCapacity DTO.
        :rtype: dict
        :example return: {"value": 100, "unit": "JoulePerCubicMeterKelvin"}
        """
        return {"value": self.value, "unit": self.unit.value}

    @staticmethod
    def from_json(data):
        """
        Obtain a new instance of VolumetricHeatCapacity DTO from a json representation.

        :param data: The VolumetricHeatCapacity DTO in JSON representation.
        :type data: dict
        :example data: {"value": 100, "unit": "JoulePerCubicMeterKelvin"}
        :return: A new instance of VolumetricHeatCapacityDto.
        :rtype: VolumetricHeatCapacityDto
        """
        return VolumetricHeatCapacityDto(value=data["value"], unit=VolumetricHeatCapacityUnits(data["unit"]))


class VolumetricHeatCapacity(AbstractMeasure):
    """
    The volumetric heat capacity is the amount of energy that must be added, in the form of heat, to one unit of volume of the material in order to cause an increase of one unit in its temperature.

    Args:
        value (float): The value.
        from_unit (VolumetricHeatCapacityUnits): The VolumetricHeatCapacity unit to create from, The default unit is JoulePerCubicMeterKelvin
    """
    def __init__(self, value: float, from_unit: VolumetricHeatCapacityUnits = VolumetricHeatCapacityUnits.JoulePerCubicMeterKelvin):
        # Do not validate type, to allow working with numpay arrays and similar objects who supports all arithmetic 
        # operations, but they are not a number, see #14 
        # if math.isnan(value):
        #     raise ValueError('Invalid unit: value is NaN')
        self._value = self.__convert_to_base(value, from_unit)
        
        self.__joules_per_cubic_meter_kelvin = None
        
        self.__joules_per_cubic_meter_degree_celsius = None
        
        self.__calories_per_cubic_centimeter_degree_celsius = None
        
        self.__btus_per_cubic_foot_degree_fahrenheit = None
        
        self.__kilojoules_per_cubic_meter_kelvin = None
        
        self.__megajoules_per_cubic_meter_kelvin = None
        
        self.__kilojoules_per_cubic_meter_degree_celsius = None
        
        self.__megajoules_per_cubic_meter_degree_celsius = None
        
        self.__kilocalories_per_cubic_centimeter_degree_celsius = None
        

    def convert(self, unit: VolumetricHeatCapacityUnits) -> float:
        return self.__convert_from_base(unit)

    def to_dto(self, hold_in_unit: VolumetricHeatCapacityUnits = VolumetricHeatCapacityUnits.JoulePerCubicMeterKelvin) -> VolumetricHeatCapacityDto:
        """
        Get a new instance of VolumetricHeatCapacity DTO representing the current unit.

        :param hold_in_unit: The specific VolumetricHeatCapacity unit to store the VolumetricHeatCapacity value in the DTO representation.
        :type hold_in_unit: VolumetricHeatCapacityUnits
        :return: A new instance of VolumetricHeatCapacityDto.
        :rtype: VolumetricHeatCapacityDto
        """
        return VolumetricHeatCapacityDto(value=self.convert(hold_in_unit), unit=hold_in_unit)
    
    def to_dto_json(self, hold_in_unit: VolumetricHeatCapacityUnits = VolumetricHeatCapacityUnits.JoulePerCubicMeterKelvin):
        """
        Get a VolumetricHeatCapacity DTO JSON object representing the current unit.

        :param hold_in_unit: The specific VolumetricHeatCapacity unit to store the VolumetricHeatCapacity value in the DTO representation.
        :type hold_in_unit: VolumetricHeatCapacityUnits
        :return: JSON object represents VolumetricHeatCapacity DTO.
        :rtype: dict
        :example return: {"value": 100, "unit": "JoulePerCubicMeterKelvin"}
        """
        return self.to_dto(hold_in_unit).to_json()

    @staticmethod
    def from_dto(volumetric_heat_capacity_dto: VolumetricHeatCapacityDto):
        """
        Obtain a new instance of VolumetricHeatCapacity from a DTO unit object.

        :param volumetric_heat_capacity_dto: The VolumetricHeatCapacity DTO representation.
        :type volumetric_heat_capacity_dto: VolumetricHeatCapacityDto
        :return: A new instance of VolumetricHeatCapacity.
        :rtype: VolumetricHeatCapacity
        """
        return VolumetricHeatCapacity(volumetric_heat_capacity_dto.value, volumetric_heat_capacity_dto.unit)

    @staticmethod
    def from_dto_json(data: dict):
        """
        Obtain a new instance of VolumetricHeatCapacity from a DTO unit json representation.

        :param data: The VolumetricHeatCapacity DTO in JSON representation.
        :type data: dict
        :example data: {"value": 100, "unit": "JoulePerCubicMeterKelvin"}
        :return: A new instance of VolumetricHeatCapacity.
        :rtype: VolumetricHeatCapacity
        """
        return VolumetricHeatCapacity.from_dto(VolumetricHeatCapacityDto.from_json(data))

    def __convert_from_base(self, from_unit: VolumetricHeatCapacityUnits) -> float:
        value = self._value
        
        if from_unit == VolumetricHeatCapacityUnits.JoulePerCubicMeterKelvin:
            return (value)
        
        if from_unit == VolumetricHeatCapacityUnits.JoulePerCubicMeterDegreeCelsius:
            return (value)
        
        if from_unit == VolumetricHeatCapacityUnits.CaloriePerCubicCentimeterDegreeCelsius:
            return (value * 2.388459e-7)
        
        if from_unit == VolumetricHeatCapacityUnits.BtuPerCubicFootDegreeFahrenheit:
            return (value * 1.4910660e-5)
        
        if from_unit == VolumetricHeatCapacityUnits.KilojoulePerCubicMeterKelvin:
            return ((value) / 1000.0)
        
        if from_unit == VolumetricHeatCapacityUnits.MegajoulePerCubicMeterKelvin:
            return ((value) / 1000000.0)
        
        if from_unit == VolumetricHeatCapacityUnits.KilojoulePerCubicMeterDegreeCelsius:
            return ((value) / 1000.0)
        
        if from_unit == VolumetricHeatCapacityUnits.MegajoulePerCubicMeterDegreeCelsius:
            return ((value) / 1000000.0)
        
        if from_unit == VolumetricHeatCapacityUnits.KilocaloriePerCubicCentimeterDegreeCelsius:
            return ((value * 2.388459e-7) / 1000.0)
        
        return None


    def __convert_to_base(self, value: float, to_unit: VolumetricHeatCapacityUnits) -> float:
        
        if to_unit == VolumetricHeatCapacityUnits.JoulePerCubicMeterKelvin:
            return (value)
        
        if to_unit == VolumetricHeatCapacityUnits.JoulePerCubicMeterDegreeCelsius:
            return (value)
        
        if to_unit == VolumetricHeatCapacityUnits.CaloriePerCubicCentimeterDegreeCelsius:
            return (value / 2.388459e-7)
        
        if to_unit == VolumetricHeatCapacityUnits.BtuPerCubicFootDegreeFahrenheit:
            return (value / 1.4910660e-5)
        
        if to_unit == VolumetricHeatCapacityUnits.KilojoulePerCubicMeterKelvin:
            return ((value) * 1000.0)
        
        if to_unit == VolumetricHeatCapacityUnits.MegajoulePerCubicMeterKelvin:
            return ((value) * 1000000.0)
        
        if to_unit == VolumetricHeatCapacityUnits.KilojoulePerCubicMeterDegreeCelsius:
            return ((value) * 1000.0)
        
        if to_unit == VolumetricHeatCapacityUnits.MegajoulePerCubicMeterDegreeCelsius:
            return ((value) * 1000000.0)
        
        if to_unit == VolumetricHeatCapacityUnits.KilocaloriePerCubicCentimeterDegreeCelsius:
            return ((value / 2.388459e-7) * 1000.0)
        
        return None


    @property
    def base_value(self) -> float:
        return self._value

    
    @staticmethod
    def from_joules_per_cubic_meter_kelvin(joules_per_cubic_meter_kelvin: float):
        """
        Create a new instance of VolumetricHeatCapacity from a value in joules_per_cubic_meter_kelvin.

        

        :param meters: The VolumetricHeatCapacity value in joules_per_cubic_meter_kelvin.
        :type joules_per_cubic_meter_kelvin: float
        :return: A new instance of VolumetricHeatCapacity.
        :rtype: VolumetricHeatCapacity
        """
        return VolumetricHeatCapacity(joules_per_cubic_meter_kelvin, VolumetricHeatCapacityUnits.JoulePerCubicMeterKelvin)

    
    @staticmethod
    def from_joules_per_cubic_meter_degree_celsius(joules_per_cubic_meter_degree_celsius: float):
        """
        Create a new instance of VolumetricHeatCapacity from a value in joules_per_cubic_meter_degree_celsius.

        

        :param meters: The VolumetricHeatCapacity value in joules_per_cubic_meter_degree_celsius.
        :type joules_per_cubic_meter_degree_celsius: float
        :return: A new instance of VolumetricHeatCapacity.
        :rtype: VolumetricHeatCapacity
        """
        return VolumetricHeatCapacity(joules_per_cubic_meter_degree_celsius, VolumetricHeatCapacityUnits.JoulePerCubicMeterDegreeCelsius)

    
    @staticmethod
    def from_calories_per_cubic_centimeter_degree_celsius(calories_per_cubic_centimeter_degree_celsius: float):
        """
        Create a new instance of VolumetricHeatCapacity from a value in calories_per_cubic_centimeter_degree_celsius.

        

        :param meters: The VolumetricHeatCapacity value in calories_per_cubic_centimeter_degree_celsius.
        :type calories_per_cubic_centimeter_degree_celsius: float
        :return: A new instance of VolumetricHeatCapacity.
        :rtype: VolumetricHeatCapacity
        """
        return VolumetricHeatCapacity(calories_per_cubic_centimeter_degree_celsius, VolumetricHeatCapacityUnits.CaloriePerCubicCentimeterDegreeCelsius)

    
    @staticmethod
    def from_btus_per_cubic_foot_degree_fahrenheit(btus_per_cubic_foot_degree_fahrenheit: float):
        """
        Create a new instance of VolumetricHeatCapacity from a value in btus_per_cubic_foot_degree_fahrenheit.

        

        :param meters: The VolumetricHeatCapacity value in btus_per_cubic_foot_degree_fahrenheit.
        :type btus_per_cubic_foot_degree_fahrenheit: float
        :return: A new instance of VolumetricHeatCapacity.
        :rtype: VolumetricHeatCapacity
        """
        return VolumetricHeatCapacity(btus_per_cubic_foot_degree_fahrenheit, VolumetricHeatCapacityUnits.BtuPerCubicFootDegreeFahrenheit)

    
    @staticmethod
    def from_kilojoules_per_cubic_meter_kelvin(kilojoules_per_cubic_meter_kelvin: float):
        """
        Create a new instance of VolumetricHeatCapacity from a value in kilojoules_per_cubic_meter_kelvin.

        

        :param meters: The VolumetricHeatCapacity value in kilojoules_per_cubic_meter_kelvin.
        :type kilojoules_per_cubic_meter_kelvin: float
        :return: A new instance of VolumetricHeatCapacity.
        :rtype: VolumetricHeatCapacity
        """
        return VolumetricHeatCapacity(kilojoules_per_cubic_meter_kelvin, VolumetricHeatCapacityUnits.KilojoulePerCubicMeterKelvin)

    
    @staticmethod
    def from_megajoules_per_cubic_meter_kelvin(megajoules_per_cubic_meter_kelvin: float):
        """
        Create a new instance of VolumetricHeatCapacity from a value in megajoules_per_cubic_meter_kelvin.

        

        :param meters: The VolumetricHeatCapacity value in megajoules_per_cubic_meter_kelvin.
        :type megajoules_per_cubic_meter_kelvin: float
        :return: A new instance of VolumetricHeatCapacity.
        :rtype: VolumetricHeatCapacity
        """
        return VolumetricHeatCapacity(megajoules_per_cubic_meter_kelvin, VolumetricHeatCapacityUnits.MegajoulePerCubicMeterKelvin)

    
    @staticmethod
    def from_kilojoules_per_cubic_meter_degree_celsius(kilojoules_per_cubic_meter_degree_celsius: float):
        """
        Create a new instance of VolumetricHeatCapacity from a value in kilojoules_per_cubic_meter_degree_celsius.

        

        :param meters: The VolumetricHeatCapacity value in kilojoules_per_cubic_meter_degree_celsius.
        :type kilojoules_per_cubic_meter_degree_celsius: float
        :return: A new instance of VolumetricHeatCapacity.
        :rtype: VolumetricHeatCapacity
        """
        return VolumetricHeatCapacity(kilojoules_per_cubic_meter_degree_celsius, VolumetricHeatCapacityUnits.KilojoulePerCubicMeterDegreeCelsius)

    
    @staticmethod
    def from_megajoules_per_cubic_meter_degree_celsius(megajoules_per_cubic_meter_degree_celsius: float):
        """
        Create a new instance of VolumetricHeatCapacity from a value in megajoules_per_cubic_meter_degree_celsius.

        

        :param meters: The VolumetricHeatCapacity value in megajoules_per_cubic_meter_degree_celsius.
        :type megajoules_per_cubic_meter_degree_celsius: float
        :return: A new instance of VolumetricHeatCapacity.
        :rtype: VolumetricHeatCapacity
        """
        return VolumetricHeatCapacity(megajoules_per_cubic_meter_degree_celsius, VolumetricHeatCapacityUnits.MegajoulePerCubicMeterDegreeCelsius)

    
    @staticmethod
    def from_kilocalories_per_cubic_centimeter_degree_celsius(kilocalories_per_cubic_centimeter_degree_celsius: float):
        """
        Create a new instance of VolumetricHeatCapacity from a value in kilocalories_per_cubic_centimeter_degree_celsius.

        

        :param meters: The VolumetricHeatCapacity value in kilocalories_per_cubic_centimeter_degree_celsius.
        :type kilocalories_per_cubic_centimeter_degree_celsius: float
        :return: A new instance of VolumetricHeatCapacity.
        :rtype: VolumetricHeatCapacity
        """
        return VolumetricHeatCapacity(kilocalories_per_cubic_centimeter_degree_celsius, VolumetricHeatCapacityUnits.KilocaloriePerCubicCentimeterDegreeCelsius)

    
    @property
    def joules_per_cubic_meter_kelvin(self) -> float:
        """
        
        """
        if self.__joules_per_cubic_meter_kelvin != None:
            return self.__joules_per_cubic_meter_kelvin
        self.__joules_per_cubic_meter_kelvin = self.__convert_from_base(VolumetricHeatCapacityUnits.JoulePerCubicMeterKelvin)
        return self.__joules_per_cubic_meter_kelvin

    
    @property
    def joules_per_cubic_meter_degree_celsius(self) -> float:
        """
        
        """
        if self.__joules_per_cubic_meter_degree_celsius != None:
            return self.__joules_per_cubic_meter_degree_celsius
        self.__joules_per_cubic_meter_degree_celsius = self.__convert_from_base(VolumetricHeatCapacityUnits.JoulePerCubicMeterDegreeCelsius)
        return self.__joules_per_cubic_meter_degree_celsius

    
    @property
    def calories_per_cubic_centimeter_degree_celsius(self) -> float:
        """
        
        """
        if self.__calories_per_cubic_centimeter_degree_celsius != None:
            return self.__calories_per_cubic_centimeter_degree_celsius
        self.__calories_per_cubic_centimeter_degree_celsius = self.__convert_from_base(VolumetricHeatCapacityUnits.CaloriePerCubicCentimeterDegreeCelsius)
        return self.__calories_per_cubic_centimeter_degree_celsius

    
    @property
    def btus_per_cubic_foot_degree_fahrenheit(self) -> float:
        """
        
        """
        if self.__btus_per_cubic_foot_degree_fahrenheit != None:
            return self.__btus_per_cubic_foot_degree_fahrenheit
        self.__btus_per_cubic_foot_degree_fahrenheit = self.__convert_from_base(VolumetricHeatCapacityUnits.BtuPerCubicFootDegreeFahrenheit)
        return self.__btus_per_cubic_foot_degree_fahrenheit

    
    @property
    def kilojoules_per_cubic_meter_kelvin(self) -> float:
        """
        
        """
        if self.__kilojoules_per_cubic_meter_kelvin != None:
            return self.__kilojoules_per_cubic_meter_kelvin
        self.__kilojoules_per_cubic_meter_kelvin = self.__convert_from_base(VolumetricHeatCapacityUnits.KilojoulePerCubicMeterKelvin)
        return self.__kilojoules_per_cubic_meter_kelvin

    
    @property
    def megajoules_per_cubic_meter_kelvin(self) -> float:
        """
        
        """
        if self.__megajoules_per_cubic_meter_kelvin != None:
            return self.__megajoules_per_cubic_meter_kelvin
        self.__megajoules_per_cubic_meter_kelvin = self.__convert_from_base(VolumetricHeatCapacityUnits.MegajoulePerCubicMeterKelvin)
        return self.__megajoules_per_cubic_meter_kelvin

    
    @property
    def kilojoules_per_cubic_meter_degree_celsius(self) -> float:
        """
        
        """
        if self.__kilojoules_per_cubic_meter_degree_celsius != None:
            return self.__kilojoules_per_cubic_meter_degree_celsius
        self.__kilojoules_per_cubic_meter_degree_celsius = self.__convert_from_base(VolumetricHeatCapacityUnits.KilojoulePerCubicMeterDegreeCelsius)
        return self.__kilojoules_per_cubic_meter_degree_celsius

    
    @property
    def megajoules_per_cubic_meter_degree_celsius(self) -> float:
        """
        
        """
        if self.__megajoules_per_cubic_meter_degree_celsius != None:
            return self.__megajoules_per_cubic_meter_degree_celsius
        self.__megajoules_per_cubic_meter_degree_celsius = self.__convert_from_base(VolumetricHeatCapacityUnits.MegajoulePerCubicMeterDegreeCelsius)
        return self.__megajoules_per_cubic_meter_degree_celsius

    
    @property
    def kilocalories_per_cubic_centimeter_degree_celsius(self) -> float:
        """
        
        """
        if self.__kilocalories_per_cubic_centimeter_degree_celsius != None:
            return self.__kilocalories_per_cubic_centimeter_degree_celsius
        self.__kilocalories_per_cubic_centimeter_degree_celsius = self.__convert_from_base(VolumetricHeatCapacityUnits.KilocaloriePerCubicCentimeterDegreeCelsius)
        return self.__kilocalories_per_cubic_centimeter_degree_celsius

    
    def to_string(self, unit: VolumetricHeatCapacityUnits = VolumetricHeatCapacityUnits.JoulePerCubicMeterKelvin, fractional_digits: int = None) -> str:
        """
        Format the VolumetricHeatCapacity to a string.
        
        Note: the default format for VolumetricHeatCapacity is JoulePerCubicMeterKelvin.
        To specify the unit format, set the 'unit' parameter.
        
        Args:
            unit (str): The unit to format the VolumetricHeatCapacity. Default is 'JoulePerCubicMeterKelvin'.
            fractional_digits (int, optional): The number of fractional digits to keep.

        Returns:
            str: The string format of the Angle.
        """
        
        if unit == VolumetricHeatCapacityUnits.JoulePerCubicMeterKelvin:
            return f"""{super()._truncate_fraction_digits(self.joules_per_cubic_meter_kelvin, fractional_digits)} J/m³·K"""
        
        if unit == VolumetricHeatCapacityUnits.JoulePerCubicMeterDegreeCelsius:
            return f"""{super()._truncate_fraction_digits(self.joules_per_cubic_meter_degree_celsius, fractional_digits)} J/m³·°C"""
        
        if unit == VolumetricHeatCapacityUnits.CaloriePerCubicCentimeterDegreeCelsius:
            return f"""{super()._truncate_fraction_digits(self.calories_per_cubic_centimeter_degree_celsius, fractional_digits)} cal/cm³·°C"""
        
        if unit == VolumetricHeatCapacityUnits.BtuPerCubicFootDegreeFahrenheit:
            return f"""{super()._truncate_fraction_digits(self.btus_per_cubic_foot_degree_fahrenheit, fractional_digits)} BTU/ft³·°F"""
        
        if unit == VolumetricHeatCapacityUnits.KilojoulePerCubicMeterKelvin:
            return f"""{super()._truncate_fraction_digits(self.kilojoules_per_cubic_meter_kelvin, fractional_digits)} kJ/m³·K"""
        
        if unit == VolumetricHeatCapacityUnits.MegajoulePerCubicMeterKelvin:
            return f"""{super()._truncate_fraction_digits(self.megajoules_per_cubic_meter_kelvin, fractional_digits)} MJ/m³·K"""
        
        if unit == VolumetricHeatCapacityUnits.KilojoulePerCubicMeterDegreeCelsius:
            return f"""{super()._truncate_fraction_digits(self.kilojoules_per_cubic_meter_degree_celsius, fractional_digits)} kJ/m³·°C"""
        
        if unit == VolumetricHeatCapacityUnits.MegajoulePerCubicMeterDegreeCelsius:
            return f"""{super()._truncate_fraction_digits(self.megajoules_per_cubic_meter_degree_celsius, fractional_digits)} MJ/m³·°C"""
        
        if unit == VolumetricHeatCapacityUnits.KilocaloriePerCubicCentimeterDegreeCelsius:
            return f"""{super()._truncate_fraction_digits(self.kilocalories_per_cubic_centimeter_degree_celsius, fractional_digits)} kcal/cm³·°C"""
        
        return f'{self._value}'


    def get_unit_abbreviation(self, unit_abbreviation: VolumetricHeatCapacityUnits = VolumetricHeatCapacityUnits.JoulePerCubicMeterKelvin) -> str:
        """
        Get VolumetricHeatCapacity unit abbreviation.
        Note! the default abbreviation for VolumetricHeatCapacity is JoulePerCubicMeterKelvin.
        To specify the unit abbreviation set the 'unit_abbreviation' parameter.
        """
        
        if unit_abbreviation == VolumetricHeatCapacityUnits.JoulePerCubicMeterKelvin:
            return """J/m³·K"""
        
        if unit_abbreviation == VolumetricHeatCapacityUnits.JoulePerCubicMeterDegreeCelsius:
            return """J/m³·°C"""
        
        if unit_abbreviation == VolumetricHeatCapacityUnits.CaloriePerCubicCentimeterDegreeCelsius:
            return """cal/cm³·°C"""
        
        if unit_abbreviation == VolumetricHeatCapacityUnits.BtuPerCubicFootDegreeFahrenheit:
            return """BTU/ft³·°F"""
        
        if unit_abbreviation == VolumetricHeatCapacityUnits.KilojoulePerCubicMeterKelvin:
            return """kJ/m³·K"""
        
        if unit_abbreviation == VolumetricHeatCapacityUnits.MegajoulePerCubicMeterKelvin:
            return """MJ/m³·K"""
        
        if unit_abbreviation == VolumetricHeatCapacityUnits.KilojoulePerCubicMeterDegreeCelsius:
            return """kJ/m³·°C"""
        
        if unit_abbreviation == VolumetricHeatCapacityUnits.MegajoulePerCubicMeterDegreeCelsius:
            return """MJ/m³·°C"""
        
        if unit_abbreviation == VolumetricHeatCapacityUnits.KilocaloriePerCubicCentimeterDegreeCelsius:
            return """kcal/cm³·°C"""
        


from enum import Enum
import math

from ..abstract_unit import AbstractMeasure



class VolumeFlowPerAreaUnits(Enum):
        """
            VolumeFlowPerAreaUnits enumeration
        """
        
        CubicMeterPerSecondPerSquareMeter = 'CubicMeterPerSecondPerSquareMeter'
        """
            
        """
        
        CubicFootPerMinutePerSquareFoot = 'CubicFootPerMinutePerSquareFoot'
        """
            
        """
        

class VolumeFlowPerAreaDto:
    """
    A DTO representation of a VolumeFlowPerArea

    Attributes:
        value (float): The value of the VolumeFlowPerArea.
        unit (VolumeFlowPerAreaUnits): The specific unit that the VolumeFlowPerArea value is representing.
    """

    def __init__(self, value: float, unit: VolumeFlowPerAreaUnits):
        """
        Create a new DTO representation of a VolumeFlowPerArea

        Parameters:
            value (float): The value of the VolumeFlowPerArea.
            unit (VolumeFlowPerAreaUnits): The specific unit that the VolumeFlowPerArea value is representing.
        """
        self.value: float = value
        """
        The value of the VolumeFlowPerArea
        """
        self.unit: VolumeFlowPerAreaUnits = unit
        """
        The specific unit that the VolumeFlowPerArea value is representing
        """

    def to_json(self):
        """
        Get a VolumeFlowPerArea DTO JSON object representing the current unit.

        :return: JSON object represents VolumeFlowPerArea DTO.
        :rtype: dict
        :example return: {"value": 100, "unit": "CubicMeterPerSecondPerSquareMeter"}
        """
        return {"value": self.value, "unit": self.unit.value}

    @staticmethod
    def from_json(data):
        """
        Obtain a new instance of VolumeFlowPerArea DTO from a json representation.

        :param data: The VolumeFlowPerArea DTO in JSON representation.
        :type data: dict
        :example data: {"value": 100, "unit": "CubicMeterPerSecondPerSquareMeter"}
        :return: A new instance of VolumeFlowPerAreaDto.
        :rtype: VolumeFlowPerAreaDto
        """
        return VolumeFlowPerAreaDto(value=data["value"], unit=VolumeFlowPerAreaUnits(data["unit"]))


class VolumeFlowPerArea(AbstractMeasure):
    """
    None

    Args:
        value (float): The value.
        from_unit (VolumeFlowPerAreaUnits): The VolumeFlowPerArea unit to create from, The default unit is CubicMeterPerSecondPerSquareMeter
    """
    def __init__(self, value: float, from_unit: VolumeFlowPerAreaUnits = VolumeFlowPerAreaUnits.CubicMeterPerSecondPerSquareMeter):
        # Do not validate type, to allow working with numpay arrays and similar objects who supports all arithmetic 
        # operations, but they are not a number, see #14 
        # if math.isnan(value):
        #     raise ValueError('Invalid unit: value is NaN')
        self._value = self.__convert_to_base(value, from_unit)
        
        self.__cubic_meters_per_second_per_square_meter = None
        
        self.__cubic_feet_per_minute_per_square_foot = None
        

    def convert(self, unit: VolumeFlowPerAreaUnits) -> float:
        return self.__convert_from_base(unit)

    def to_dto(self, hold_in_unit: VolumeFlowPerAreaUnits = VolumeFlowPerAreaUnits.CubicMeterPerSecondPerSquareMeter) -> VolumeFlowPerAreaDto:
        """
        Get a new instance of VolumeFlowPerArea DTO representing the current unit.

        :param hold_in_unit: The specific VolumeFlowPerArea unit to store the VolumeFlowPerArea value in the DTO representation.
        :type hold_in_unit: VolumeFlowPerAreaUnits
        :return: A new instance of VolumeFlowPerAreaDto.
        :rtype: VolumeFlowPerAreaDto
        """
        return VolumeFlowPerAreaDto(value=self.convert(hold_in_unit), unit=hold_in_unit)
    
    def to_dto_json(self, hold_in_unit: VolumeFlowPerAreaUnits = VolumeFlowPerAreaUnits.CubicMeterPerSecondPerSquareMeter):
        """
        Get a VolumeFlowPerArea DTO JSON object representing the current unit.

        :param hold_in_unit: The specific VolumeFlowPerArea unit to store the VolumeFlowPerArea value in the DTO representation.
        :type hold_in_unit: VolumeFlowPerAreaUnits
        :return: JSON object represents VolumeFlowPerArea DTO.
        :rtype: dict
        :example return: {"value": 100, "unit": "CubicMeterPerSecondPerSquareMeter"}
        """
        return self.to_dto(hold_in_unit).to_json()

    @staticmethod
    def from_dto(volume_flow_per_area_dto: VolumeFlowPerAreaDto):
        """
        Obtain a new instance of VolumeFlowPerArea from a DTO unit object.

        :param volume_flow_per_area_dto: The VolumeFlowPerArea DTO representation.
        :type volume_flow_per_area_dto: VolumeFlowPerAreaDto
        :return: A new instance of VolumeFlowPerArea.
        :rtype: VolumeFlowPerArea
        """
        return VolumeFlowPerArea(volume_flow_per_area_dto.value, volume_flow_per_area_dto.unit)

    @staticmethod
    def from_dto_json(data: dict):
        """
        Obtain a new instance of VolumeFlowPerArea from a DTO unit json representation.

        :param data: The VolumeFlowPerArea DTO in JSON representation.
        :type data: dict
        :example data: {"value": 100, "unit": "CubicMeterPerSecondPerSquareMeter"}
        :return: A new instance of VolumeFlowPerArea.
        :rtype: VolumeFlowPerArea
        """
        return VolumeFlowPerArea.from_dto(VolumeFlowPerAreaDto.from_json(data))

    def __convert_from_base(self, from_unit: VolumeFlowPerAreaUnits) -> float:
        value = self._value
        
        if from_unit == VolumeFlowPerAreaUnits.CubicMeterPerSecondPerSquareMeter:
            return (value)
        
        if from_unit == VolumeFlowPerAreaUnits.CubicFootPerMinutePerSquareFoot:
            return (value * 196.850394)
        
        return None


    def __convert_to_base(self, value: float, to_unit: VolumeFlowPerAreaUnits) -> float:
        
        if to_unit == VolumeFlowPerAreaUnits.CubicMeterPerSecondPerSquareMeter:
            return (value)
        
        if to_unit == VolumeFlowPerAreaUnits.CubicFootPerMinutePerSquareFoot:
            return (value / 196.850394)
        
        return None


    @property
    def base_value(self) -> float:
        return self._value

    
    @staticmethod
    def from_cubic_meters_per_second_per_square_meter(cubic_meters_per_second_per_square_meter: float):
        """
        Create a new instance of VolumeFlowPerArea from a value in cubic_meters_per_second_per_square_meter.

        

        :param meters: The VolumeFlowPerArea value in cubic_meters_per_second_per_square_meter.
        :type cubic_meters_per_second_per_square_meter: float
        :return: A new instance of VolumeFlowPerArea.
        :rtype: VolumeFlowPerArea
        """
        return VolumeFlowPerArea(cubic_meters_per_second_per_square_meter, VolumeFlowPerAreaUnits.CubicMeterPerSecondPerSquareMeter)

    
    @staticmethod
    def from_cubic_feet_per_minute_per_square_foot(cubic_feet_per_minute_per_square_foot: float):
        """
        Create a new instance of VolumeFlowPerArea from a value in cubic_feet_per_minute_per_square_foot.

        

        :param meters: The VolumeFlowPerArea value in cubic_feet_per_minute_per_square_foot.
        :type cubic_feet_per_minute_per_square_foot: float
        :return: A new instance of VolumeFlowPerArea.
        :rtype: VolumeFlowPerArea
        """
        return VolumeFlowPerArea(cubic_feet_per_minute_per_square_foot, VolumeFlowPerAreaUnits.CubicFootPerMinutePerSquareFoot)

    
    @property
    def cubic_meters_per_second_per_square_meter(self) -> float:
        """
        
        """
        if self.__cubic_meters_per_second_per_square_meter != None:
            return self.__cubic_meters_per_second_per_square_meter
        self.__cubic_meters_per_second_per_square_meter = self.__convert_from_base(VolumeFlowPerAreaUnits.CubicMeterPerSecondPerSquareMeter)
        return self.__cubic_meters_per_second_per_square_meter

    
    @property
    def cubic_feet_per_minute_per_square_foot(self) -> float:
        """
        
        """
        if self.__cubic_feet_per_minute_per_square_foot != None:
            return self.__cubic_feet_per_minute_per_square_foot
        self.__cubic_feet_per_minute_per_square_foot = self.__convert_from_base(VolumeFlowPerAreaUnits.CubicFootPerMinutePerSquareFoot)
        return self.__cubic_feet_per_minute_per_square_foot

    
    def to_string(self, unit: VolumeFlowPerAreaUnits = VolumeFlowPerAreaUnits.CubicMeterPerSecondPerSquareMeter, fractional_digits: int = None) -> str:
        """
        Format the VolumeFlowPerArea to a string.
        
        Note: the default format for VolumeFlowPerArea is CubicMeterPerSecondPerSquareMeter.
        To specify the unit format, set the 'unit' parameter.
        
        Args:
            unit (str): The unit to format the VolumeFlowPerArea. Default is 'CubicMeterPerSecondPerSquareMeter'.
            fractional_digits (int, optional): The number of fractional digits to keep.

        Returns:
            str: The string format of the Angle.
        """
        
        if unit == VolumeFlowPerAreaUnits.CubicMeterPerSecondPerSquareMeter:
            return f"""{super()._truncate_fraction_digits(self.cubic_meters_per_second_per_square_meter, fractional_digits)} m³/(s·m²)"""
        
        if unit == VolumeFlowPerAreaUnits.CubicFootPerMinutePerSquareFoot:
            return f"""{super()._truncate_fraction_digits(self.cubic_feet_per_minute_per_square_foot, fractional_digits)} CFM/ft²"""
        
        return f'{self._value}'


    def get_unit_abbreviation(self, unit_abbreviation: VolumeFlowPerAreaUnits = VolumeFlowPerAreaUnits.CubicMeterPerSecondPerSquareMeter) -> str:
        """
        Get VolumeFlowPerArea unit abbreviation.
        Note! the default abbreviation for VolumeFlowPerArea is CubicMeterPerSecondPerSquareMeter.
        To specify the unit abbreviation set the 'unit_abbreviation' parameter.
        """
        
        if unit_abbreviation == VolumeFlowPerAreaUnits.CubicMeterPerSecondPerSquareMeter:
            return """m³/(s·m²)"""
        
        if unit_abbreviation == VolumeFlowPerAreaUnits.CubicFootPerMinutePerSquareFoot:
            return """CFM/ft²"""
        


from enum import Enum
import math

from ..abstract_unit import AbstractMeasure



class RatioUnits(Enum):
        """
            RatioUnits enumeration
        """
        
        DecimalFraction = 'DecimalFraction'
        """
            
        """
        
        Percent = 'Percent'
        """
            
        """
        
        PartPerThousand = 'PartPerThousand'
        """
            
        """
        
        PartPerMillion = 'PartPerMillion'
        """
            
        """
        
        PartPerBillion = 'PartPerBillion'
        """
            
        """
        
        PartPerTrillion = 'PartPerTrillion'
        """
            
        """
        

class RatioDto:
    """
    A DTO representation of a Ratio

    Attributes:
        value (float): The value of the Ratio.
        unit (RatioUnits): The specific unit that the Ratio value is representing.
    """

    def __init__(self, value: float, unit: RatioUnits):
        """
        Create a new DTO representation of a Ratio

        Parameters:
            value (float): The value of the Ratio.
            unit (RatioUnits): The specific unit that the Ratio value is representing.
        """
        self.value: float = value
        """
        The value of the Ratio
        """
        self.unit: RatioUnits = unit
        """
        The specific unit that the Ratio value is representing
        """

    def to_json(self):
        """
        Get a Ratio DTO JSON object representing the current unit.

        :return: JSON object represents Ratio DTO.
        :rtype: dict
        :example return: {"value": 100, "unit": "DecimalFraction"}
        """
        return {"value": self.value, "unit": self.unit.value}

    @staticmethod
    def from_json(data):
        """
        Obtain a new instance of Ratio DTO from a json representation.

        :param data: The Ratio DTO in JSON representation.
        :type data: dict
        :example data: {"value": 100, "unit": "DecimalFraction"}
        :return: A new instance of RatioDto.
        :rtype: RatioDto
        """
        return RatioDto(value=data["value"], unit=RatioUnits(data["unit"]))


class Ratio(AbstractMeasure):
    """
    In mathematics, a ratio is a relationship between two numbers of the same kind (e.g., objects, persons, students, spoonfuls, units of whatever identical dimension), usually expressed as "a to b" or a:b, sometimes expressed arithmetically as a dimensionless quotient of the two that explicitly indicates how many times the first number contains the second (not necessarily an integer).

    Args:
        value (float): The value.
        from_unit (RatioUnits): The Ratio unit to create from, The default unit is DecimalFraction
    """
    def __init__(self, value: float, from_unit: RatioUnits = RatioUnits.DecimalFraction):
        # Do not validate type, to allow working with numpay arrays and similar objects who supports all arithmetic 
        # operations, but they are not a number, see #14 
        # if math.isnan(value):
        #     raise ValueError('Invalid unit: value is NaN')
        self._value = self.__convert_to_base(value, from_unit)
        
        self.__decimal_fractions = None
        
        self.__percent = None
        
        self.__parts_per_thousand = None
        
        self.__parts_per_million = None
        
        self.__parts_per_billion = None
        
        self.__parts_per_trillion = None
        

    def convert(self, unit: RatioUnits) -> float:
        return self.__convert_from_base(unit)

    def to_dto(self, hold_in_unit: RatioUnits = RatioUnits.DecimalFraction) -> RatioDto:
        """
        Get a new instance of Ratio DTO representing the current unit.

        :param hold_in_unit: The specific Ratio unit to store the Ratio value in the DTO representation.
        :type hold_in_unit: RatioUnits
        :return: A new instance of RatioDto.
        :rtype: RatioDto
        """
        return RatioDto(value=self.convert(hold_in_unit), unit=hold_in_unit)
    
    def to_dto_json(self, hold_in_unit: RatioUnits = RatioUnits.DecimalFraction):
        """
        Get a Ratio DTO JSON object representing the current unit.

        :param hold_in_unit: The specific Ratio unit to store the Ratio value in the DTO representation.
        :type hold_in_unit: RatioUnits
        :return: JSON object represents Ratio DTO.
        :rtype: dict
        :example return: {"value": 100, "unit": "DecimalFraction"}
        """
        return self.to_dto(hold_in_unit).to_json()

    @staticmethod
    def from_dto(ratio_dto: RatioDto):
        """
        Obtain a new instance of Ratio from a DTO unit object.

        :param ratio_dto: The Ratio DTO representation.
        :type ratio_dto: RatioDto
        :return: A new instance of Ratio.
        :rtype: Ratio
        """
        return Ratio(ratio_dto.value, ratio_dto.unit)

    @staticmethod
    def from_dto_json(data: dict):
        """
        Obtain a new instance of Ratio from a DTO unit json representation.

        :param data: The Ratio DTO in JSON representation.
        :type data: dict
        :example data: {"value": 100, "unit": "DecimalFraction"}
        :return: A new instance of Ratio.
        :rtype: Ratio
        """
        return Ratio.from_dto(RatioDto.from_json(data))

    def __convert_from_base(self, from_unit: RatioUnits) -> float:
        value = self._value
        
        if from_unit == RatioUnits.DecimalFraction:
            return (value)
        
        if from_unit == RatioUnits.Percent:
            return (value * 1e2)
        
        if from_unit == RatioUnits.PartPerThousand:
            return (value * 1e3)
        
        if from_unit == RatioUnits.PartPerMillion:
            return (value * 1e6)
        
        if from_unit == RatioUnits.PartPerBillion:
            return (value * 1e9)
        
        if from_unit == RatioUnits.PartPerTrillion:
            return (value * 1e12)
        
        return None


    def __convert_to_base(self, value: float, to_unit: RatioUnits) -> float:
        
        if to_unit == RatioUnits.DecimalFraction:
            return (value)
        
        if to_unit == RatioUnits.Percent:
            return (value / 1e2)
        
        if to_unit == RatioUnits.PartPerThousand:
            return (value / 1e3)
        
        if to_unit == RatioUnits.PartPerMillion:
            return (value / 1e6)
        
        if to_unit == RatioUnits.PartPerBillion:
            return (value / 1e9)
        
        if to_unit == RatioUnits.PartPerTrillion:
            return (value / 1e12)
        
        return None


    @property
    def base_value(self) -> float:
        return self._value

    
    @staticmethod
    def from_decimal_fractions(decimal_fractions: float):
        """
        Create a new instance of Ratio from a value in decimal_fractions.

        

        :param meters: The Ratio value in decimal_fractions.
        :type decimal_fractions: float
        :return: A new instance of Ratio.
        :rtype: Ratio
        """
        return Ratio(decimal_fractions, RatioUnits.DecimalFraction)

    
    @staticmethod
    def from_percent(percent: float):
        """
        Create a new instance of Ratio from a value in percent.

        

        :param meters: The Ratio value in percent.
        :type percent: float
        :return: A new instance of Ratio.
        :rtype: Ratio
        """
        return Ratio(percent, RatioUnits.Percent)

    
    @staticmethod
    def from_parts_per_thousand(parts_per_thousand: float):
        """
        Create a new instance of Ratio from a value in parts_per_thousand.

        

        :param meters: The Ratio value in parts_per_thousand.
        :type parts_per_thousand: float
        :return: A new instance of Ratio.
        :rtype: Ratio
        """
        return Ratio(parts_per_thousand, RatioUnits.PartPerThousand)

    
    @staticmethod
    def from_parts_per_million(parts_per_million: float):
        """
        Create a new instance of Ratio from a value in parts_per_million.

        

        :param meters: The Ratio value in parts_per_million.
        :type parts_per_million: float
        :return: A new instance of Ratio.
        :rtype: Ratio
        """
        return Ratio(parts_per_million, RatioUnits.PartPerMillion)

    
    @staticmethod
    def from_parts_per_billion(parts_per_billion: float):
        """
        Create a new instance of Ratio from a value in parts_per_billion.

        

        :param meters: The Ratio value in parts_per_billion.
        :type parts_per_billion: float
        :return: A new instance of Ratio.
        :rtype: Ratio
        """
        return Ratio(parts_per_billion, RatioUnits.PartPerBillion)

    
    @staticmethod
    def from_parts_per_trillion(parts_per_trillion: float):
        """
        Create a new instance of Ratio from a value in parts_per_trillion.

        

        :param meters: The Ratio value in parts_per_trillion.
        :type parts_per_trillion: float
        :return: A new instance of Ratio.
        :rtype: Ratio
        """
        return Ratio(parts_per_trillion, RatioUnits.PartPerTrillion)

    
    @property
    def decimal_fractions(self) -> float:
        """
        
        """
        if self.__decimal_fractions != None:
            return self.__decimal_fractions
        self.__decimal_fractions = self.__convert_from_base(RatioUnits.DecimalFraction)
        return self.__decimal_fractions

    
    @property
    def percent(self) -> float:
        """
        
        """
        if self.__percent != None:
            return self.__percent
        self.__percent = self.__convert_from_base(RatioUnits.Percent)
        return self.__percent

    
    @property
    def parts_per_thousand(self) -> float:
        """
        
        """
        if self.__parts_per_thousand != None:
            return self.__parts_per_thousand
        self.__parts_per_thousand = self.__convert_from_base(RatioUnits.PartPerThousand)
        return self.__parts_per_thousand

    
    @property
    def parts_per_million(self) -> float:
        """
        
        """
        if self.__parts_per_million != None:
            return self.__parts_per_million
        self.__parts_per_million = self.__convert_from_base(RatioUnits.PartPerMillion)
        return self.__parts_per_million

    
    @property
    def parts_per_billion(self) -> float:
        """
        
        """
        if self.__parts_per_billion != None:
            return self.__parts_per_billion
        self.__parts_per_billion = self.__convert_from_base(RatioUnits.PartPerBillion)
        return self.__parts_per_billion

    
    @property
    def parts_per_trillion(self) -> float:
        """
        
        """
        if self.__parts_per_trillion != None:
            return self.__parts_per_trillion
        self.__parts_per_trillion = self.__convert_from_base(RatioUnits.PartPerTrillion)
        return self.__parts_per_trillion

    
    def to_string(self, unit: RatioUnits = RatioUnits.DecimalFraction, fractional_digits: int = None) -> str:
        """
        Format the Ratio to a string.
        
        Note: the default format for Ratio is DecimalFraction.
        To specify the unit format, set the 'unit' parameter.
        
        Args:
            unit (str): The unit to format the Ratio. Default is 'DecimalFraction'.
            fractional_digits (int, optional): The number of fractional digits to keep.

        Returns:
            str: The string format of the Angle.
        """
        
        if unit == RatioUnits.DecimalFraction:
            return f"""{super()._truncate_fraction_digits(self.decimal_fractions, fractional_digits)} """
        
        if unit == RatioUnits.Percent:
            return f"""{super()._truncate_fraction_digits(self.percent, fractional_digits)} %"""
        
        if unit == RatioUnits.PartPerThousand:
            return f"""{super()._truncate_fraction_digits(self.parts_per_thousand, fractional_digits)} ‰"""
        
        if unit == RatioUnits.PartPerMillion:
            return f"""{super()._truncate_fraction_digits(self.parts_per_million, fractional_digits)} ppm"""
        
        if unit == RatioUnits.PartPerBillion:
            return f"""{super()._truncate_fraction_digits(self.parts_per_billion, fractional_digits)} ppb"""
        
        if unit == RatioUnits.PartPerTrillion:
            return f"""{super()._truncate_fraction_digits(self.parts_per_trillion, fractional_digits)} ppt"""
        
        return f'{self._value}'


    def get_unit_abbreviation(self, unit_abbreviation: RatioUnits = RatioUnits.DecimalFraction) -> str:
        """
        Get Ratio unit abbreviation.
        Note! the default abbreviation for Ratio is DecimalFraction.
        To specify the unit abbreviation set the 'unit_abbreviation' parameter.
        """
        
        if unit_abbreviation == RatioUnits.DecimalFraction:
            return """"""
        
        if unit_abbreviation == RatioUnits.Percent:
            return """%"""
        
        if unit_abbreviation == RatioUnits.PartPerThousand:
            return """‰"""
        
        if unit_abbreviation == RatioUnits.PartPerMillion:
            return """ppm"""
        
        if unit_abbreviation == RatioUnits.PartPerBillion:
            return """ppb"""
        
        if unit_abbreviation == RatioUnits.PartPerTrillion:
            return """ppt"""
        


from enum import Enum
import math

from ..abstract_unit import AbstractMeasure



class ElectricChargeDensityUnits(Enum):
        """
            ElectricChargeDensityUnits enumeration
        """
        
        CoulombPerCubicMeter = 'CoulombPerCubicMeter'
        """
            
        """
        

class ElectricChargeDensityDto:
    """
    A DTO representation of a ElectricChargeDensity

    Attributes:
        value (float): The value of the ElectricChargeDensity.
        unit (ElectricChargeDensityUnits): The specific unit that the ElectricChargeDensity value is representing.
    """

    def __init__(self, value: float, unit: ElectricChargeDensityUnits):
        """
        Create a new DTO representation of a ElectricChargeDensity

        Parameters:
            value (float): The value of the ElectricChargeDensity.
            unit (ElectricChargeDensityUnits): The specific unit that the ElectricChargeDensity value is representing.
        """
        self.value: float = value
        """
        The value of the ElectricChargeDensity
        """
        self.unit: ElectricChargeDensityUnits = unit
        """
        The specific unit that the ElectricChargeDensity value is representing
        """

    def to_json(self):
        """
        Get a ElectricChargeDensity DTO JSON object representing the current unit.

        :return: JSON object represents ElectricChargeDensity DTO.
        :rtype: dict
        :example return: {"value": 100, "unit": "CoulombPerCubicMeter"}
        """
        return {"value": self.value, "unit": self.unit.value}

    @staticmethod
    def from_json(data):
        """
        Obtain a new instance of ElectricChargeDensity DTO from a json representation.

        :param data: The ElectricChargeDensity DTO in JSON representation.
        :type data: dict
        :example data: {"value": 100, "unit": "CoulombPerCubicMeter"}
        :return: A new instance of ElectricChargeDensityDto.
        :rtype: ElectricChargeDensityDto
        """
        return ElectricChargeDensityDto(value=data["value"], unit=ElectricChargeDensityUnits(data["unit"]))


class ElectricChargeDensity(AbstractMeasure):
    """
    In electromagnetism, charge density is a measure of the amount of electric charge per volume.

    Args:
        value (float): The value.
        from_unit (ElectricChargeDensityUnits): The ElectricChargeDensity unit to create from, The default unit is CoulombPerCubicMeter
    """
    def __init__(self, value: float, from_unit: ElectricChargeDensityUnits = ElectricChargeDensityUnits.CoulombPerCubicMeter):
        # Do not validate type, to allow working with numpay arrays and similar objects who supports all arithmetic 
        # operations, but they are not a number, see #14 
        # if math.isnan(value):
        #     raise ValueError('Invalid unit: value is NaN')
        self._value = self.__convert_to_base(value, from_unit)
        
        self.__coulombs_per_cubic_meter = None
        

    def convert(self, unit: ElectricChargeDensityUnits) -> float:
        return self.__convert_from_base(unit)

    def to_dto(self, hold_in_unit: ElectricChargeDensityUnits = ElectricChargeDensityUnits.CoulombPerCubicMeter) -> ElectricChargeDensityDto:
        """
        Get a new instance of ElectricChargeDensity DTO representing the current unit.

        :param hold_in_unit: The specific ElectricChargeDensity unit to store the ElectricChargeDensity value in the DTO representation.
        :type hold_in_unit: ElectricChargeDensityUnits
        :return: A new instance of ElectricChargeDensityDto.
        :rtype: ElectricChargeDensityDto
        """
        return ElectricChargeDensityDto(value=self.convert(hold_in_unit), unit=hold_in_unit)
    
    def to_dto_json(self, hold_in_unit: ElectricChargeDensityUnits = ElectricChargeDensityUnits.CoulombPerCubicMeter):
        """
        Get a ElectricChargeDensity DTO JSON object representing the current unit.

        :param hold_in_unit: The specific ElectricChargeDensity unit to store the ElectricChargeDensity value in the DTO representation.
        :type hold_in_unit: ElectricChargeDensityUnits
        :return: JSON object represents ElectricChargeDensity DTO.
        :rtype: dict
        :example return: {"value": 100, "unit": "CoulombPerCubicMeter"}
        """
        return self.to_dto(hold_in_unit).to_json()

    @staticmethod
    def from_dto(electric_charge_density_dto: ElectricChargeDensityDto):
        """
        Obtain a new instance of ElectricChargeDensity from a DTO unit object.

        :param electric_charge_density_dto: The ElectricChargeDensity DTO representation.
        :type electric_charge_density_dto: ElectricChargeDensityDto
        :return: A new instance of ElectricChargeDensity.
        :rtype: ElectricChargeDensity
        """
        return ElectricChargeDensity(electric_charge_density_dto.value, electric_charge_density_dto.unit)

    @staticmethod
    def from_dto_json(data: dict):
        """
        Obtain a new instance of ElectricChargeDensity from a DTO unit json representation.

        :param data: The ElectricChargeDensity DTO in JSON representation.
        :type data: dict
        :example data: {"value": 100, "unit": "CoulombPerCubicMeter"}
        :return: A new instance of ElectricChargeDensity.
        :rtype: ElectricChargeDensity
        """
        return ElectricChargeDensity.from_dto(ElectricChargeDensityDto.from_json(data))

    def __convert_from_base(self, from_unit: ElectricChargeDensityUnits) -> float:
        value = self._value
        
        if from_unit == ElectricChargeDensityUnits.CoulombPerCubicMeter:
            return (value)
        
        return None


    def __convert_to_base(self, value: float, to_unit: ElectricChargeDensityUnits) -> float:
        
        if to_unit == ElectricChargeDensityUnits.CoulombPerCubicMeter:
            return (value)
        
        return None


    @property
    def base_value(self) -> float:
        return self._value

    
    @staticmethod
    def from_coulombs_per_cubic_meter(coulombs_per_cubic_meter: float):
        """
        Create a new instance of ElectricChargeDensity from a value in coulombs_per_cubic_meter.

        

        :param meters: The ElectricChargeDensity value in coulombs_per_cubic_meter.
        :type coulombs_per_cubic_meter: float
        :return: A new instance of ElectricChargeDensity.
        :rtype: ElectricChargeDensity
        """
        return ElectricChargeDensity(coulombs_per_cubic_meter, ElectricChargeDensityUnits.CoulombPerCubicMeter)

    
    @property
    def coulombs_per_cubic_meter(self) -> float:
        """
        
        """
        if self.__coulombs_per_cubic_meter != None:
            return self.__coulombs_per_cubic_meter
        self.__coulombs_per_cubic_meter = self.__convert_from_base(ElectricChargeDensityUnits.CoulombPerCubicMeter)
        return self.__coulombs_per_cubic_meter

    
    def to_string(self, unit: ElectricChargeDensityUnits = ElectricChargeDensityUnits.CoulombPerCubicMeter, fractional_digits: int = None) -> str:
        """
        Format the ElectricChargeDensity to a string.
        
        Note: the default format for ElectricChargeDensity is CoulombPerCubicMeter.
        To specify the unit format, set the 'unit' parameter.
        
        Args:
            unit (str): The unit to format the ElectricChargeDensity. Default is 'CoulombPerCubicMeter'.
            fractional_digits (int, optional): The number of fractional digits to keep.

        Returns:
            str: The string format of the Angle.
        """
        
        if unit == ElectricChargeDensityUnits.CoulombPerCubicMeter:
            return f"""{super()._truncate_fraction_digits(self.coulombs_per_cubic_meter, fractional_digits)} C/m³"""
        
        return f'{self._value}'


    def get_unit_abbreviation(self, unit_abbreviation: ElectricChargeDensityUnits = ElectricChargeDensityUnits.CoulombPerCubicMeter) -> str:
        """
        Get ElectricChargeDensity unit abbreviation.
        Note! the default abbreviation for ElectricChargeDensity is CoulombPerCubicMeter.
        To specify the unit abbreviation set the 'unit_abbreviation' parameter.
        """
        
        if unit_abbreviation == ElectricChargeDensityUnits.CoulombPerCubicMeter:
            return """C/m³"""
        


from enum import Enum
import math

from ..abstract_unit import AbstractMeasure



class TemperatureGradientUnits(Enum):
        """
            TemperatureGradientUnits enumeration
        """
        
        KelvinPerMeter = 'KelvinPerMeter'
        """
            
        """
        
        DegreeCelsiusPerMeter = 'DegreeCelsiusPerMeter'
        """
            
        """
        
        DegreeFahrenheitPerFoot = 'DegreeFahrenheitPerFoot'
        """
            
        """
        
        DegreeCelsiusPerKilometer = 'DegreeCelsiusPerKilometer'
        """
            
        """
        

class TemperatureGradientDto:
    """
    A DTO representation of a TemperatureGradient

    Attributes:
        value (float): The value of the TemperatureGradient.
        unit (TemperatureGradientUnits): The specific unit that the TemperatureGradient value is representing.
    """

    def __init__(self, value: float, unit: TemperatureGradientUnits):
        """
        Create a new DTO representation of a TemperatureGradient

        Parameters:
            value (float): The value of the TemperatureGradient.
            unit (TemperatureGradientUnits): The specific unit that the TemperatureGradient value is representing.
        """
        self.value: float = value
        """
        The value of the TemperatureGradient
        """
        self.unit: TemperatureGradientUnits = unit
        """
        The specific unit that the TemperatureGradient value is representing
        """

    def to_json(self):
        """
        Get a TemperatureGradient DTO JSON object representing the current unit.

        :return: JSON object represents TemperatureGradient DTO.
        :rtype: dict
        :example return: {"value": 100, "unit": "KelvinPerMeter"}
        """
        return {"value": self.value, "unit": self.unit.value}

    @staticmethod
    def from_json(data):
        """
        Obtain a new instance of TemperatureGradient DTO from a json representation.

        :param data: The TemperatureGradient DTO in JSON representation.
        :type data: dict
        :example data: {"value": 100, "unit": "KelvinPerMeter"}
        :return: A new instance of TemperatureGradientDto.
        :rtype: TemperatureGradientDto
        """
        return TemperatureGradientDto(value=data["value"], unit=TemperatureGradientUnits(data["unit"]))


class TemperatureGradient(AbstractMeasure):
    """
    None

    Args:
        value (float): The value.
        from_unit (TemperatureGradientUnits): The TemperatureGradient unit to create from, The default unit is KelvinPerMeter
    """
    def __init__(self, value: float, from_unit: TemperatureGradientUnits = TemperatureGradientUnits.KelvinPerMeter):
        # Do not validate type, to allow working with numpay arrays and similar objects who supports all arithmetic 
        # operations, but they are not a number, see #14 
        # if math.isnan(value):
        #     raise ValueError('Invalid unit: value is NaN')
        self._value = self.__convert_to_base(value, from_unit)
        
        self.__kelvins_per_meter = None
        
        self.__degrees_celcius_per_meter = None
        
        self.__degrees_fahrenheit_per_foot = None
        
        self.__degrees_celcius_per_kilometer = None
        

    def convert(self, unit: TemperatureGradientUnits) -> float:
        return self.__convert_from_base(unit)

    def to_dto(self, hold_in_unit: TemperatureGradientUnits = TemperatureGradientUnits.KelvinPerMeter) -> TemperatureGradientDto:
        """
        Get a new instance of TemperatureGradient DTO representing the current unit.

        :param hold_in_unit: The specific TemperatureGradient unit to store the TemperatureGradient value in the DTO representation.
        :type hold_in_unit: TemperatureGradientUnits
        :return: A new instance of TemperatureGradientDto.
        :rtype: TemperatureGradientDto
        """
        return TemperatureGradientDto(value=self.convert(hold_in_unit), unit=hold_in_unit)
    
    def to_dto_json(self, hold_in_unit: TemperatureGradientUnits = TemperatureGradientUnits.KelvinPerMeter):
        """
        Get a TemperatureGradient DTO JSON object representing the current unit.

        :param hold_in_unit: The specific TemperatureGradient unit to store the TemperatureGradient value in the DTO representation.
        :type hold_in_unit: TemperatureGradientUnits
        :return: JSON object represents TemperatureGradient DTO.
        :rtype: dict
        :example return: {"value": 100, "unit": "KelvinPerMeter"}
        """
        return self.to_dto(hold_in_unit).to_json()

    @staticmethod
    def from_dto(temperature_gradient_dto: TemperatureGradientDto):
        """
        Obtain a new instance of TemperatureGradient from a DTO unit object.

        :param temperature_gradient_dto: The TemperatureGradient DTO representation.
        :type temperature_gradient_dto: TemperatureGradientDto
        :return: A new instance of TemperatureGradient.
        :rtype: TemperatureGradient
        """
        return TemperatureGradient(temperature_gradient_dto.value, temperature_gradient_dto.unit)

    @staticmethod
    def from_dto_json(data: dict):
        """
        Obtain a new instance of TemperatureGradient from a DTO unit json representation.

        :param data: The TemperatureGradient DTO in JSON representation.
        :type data: dict
        :example data: {"value": 100, "unit": "KelvinPerMeter"}
        :return: A new instance of TemperatureGradient.
        :rtype: TemperatureGradient
        """
        return TemperatureGradient.from_dto(TemperatureGradientDto.from_json(data))

    def __convert_from_base(self, from_unit: TemperatureGradientUnits) -> float:
        value = self._value
        
        if from_unit == TemperatureGradientUnits.KelvinPerMeter:
            return (value)
        
        if from_unit == TemperatureGradientUnits.DegreeCelsiusPerMeter:
            return (value)
        
        if from_unit == TemperatureGradientUnits.DegreeFahrenheitPerFoot:
            return ((value * 0.3048) * 9 / 5)
        
        if from_unit == TemperatureGradientUnits.DegreeCelsiusPerKilometer:
            return (value * 1e3)
        
        return None


    def __convert_to_base(self, value: float, to_unit: TemperatureGradientUnits) -> float:
        
        if to_unit == TemperatureGradientUnits.KelvinPerMeter:
            return (value)
        
        if to_unit == TemperatureGradientUnits.DegreeCelsiusPerMeter:
            return (value)
        
        if to_unit == TemperatureGradientUnits.DegreeFahrenheitPerFoot:
            return ((value / 0.3048) * 5 / 9)
        
        if to_unit == TemperatureGradientUnits.DegreeCelsiusPerKilometer:
            return (value / 1e3)
        
        return None


    @property
    def base_value(self) -> float:
        return self._value

    
    @staticmethod
    def from_kelvins_per_meter(kelvins_per_meter: float):
        """
        Create a new instance of TemperatureGradient from a value in kelvins_per_meter.

        

        :param meters: The TemperatureGradient value in kelvins_per_meter.
        :type kelvins_per_meter: float
        :return: A new instance of TemperatureGradient.
        :rtype: TemperatureGradient
        """
        return TemperatureGradient(kelvins_per_meter, TemperatureGradientUnits.KelvinPerMeter)

    
    @staticmethod
    def from_degrees_celcius_per_meter(degrees_celcius_per_meter: float):
        """
        Create a new instance of TemperatureGradient from a value in degrees_celcius_per_meter.

        

        :param meters: The TemperatureGradient value in degrees_celcius_per_meter.
        :type degrees_celcius_per_meter: float
        :return: A new instance of TemperatureGradient.
        :rtype: TemperatureGradient
        """
        return TemperatureGradient(degrees_celcius_per_meter, TemperatureGradientUnits.DegreeCelsiusPerMeter)

    
    @staticmethod
    def from_degrees_fahrenheit_per_foot(degrees_fahrenheit_per_foot: float):
        """
        Create a new instance of TemperatureGradient from a value in degrees_fahrenheit_per_foot.

        

        :param meters: The TemperatureGradient value in degrees_fahrenheit_per_foot.
        :type degrees_fahrenheit_per_foot: float
        :return: A new instance of TemperatureGradient.
        :rtype: TemperatureGradient
        """
        return TemperatureGradient(degrees_fahrenheit_per_foot, TemperatureGradientUnits.DegreeFahrenheitPerFoot)

    
    @staticmethod
    def from_degrees_celcius_per_kilometer(degrees_celcius_per_kilometer: float):
        """
        Create a new instance of TemperatureGradient from a value in degrees_celcius_per_kilometer.

        

        :param meters: The TemperatureGradient value in degrees_celcius_per_kilometer.
        :type degrees_celcius_per_kilometer: float
        :return: A new instance of TemperatureGradient.
        :rtype: TemperatureGradient
        """
        return TemperatureGradient(degrees_celcius_per_kilometer, TemperatureGradientUnits.DegreeCelsiusPerKilometer)

    
    @property
    def kelvins_per_meter(self) -> float:
        """
        
        """
        if self.__kelvins_per_meter != None:
            return self.__kelvins_per_meter
        self.__kelvins_per_meter = self.__convert_from_base(TemperatureGradientUnits.KelvinPerMeter)
        return self.__kelvins_per_meter

    
    @property
    def degrees_celcius_per_meter(self) -> float:
        """
        
        """
        if self.__degrees_celcius_per_meter != None:
            return self.__degrees_celcius_per_meter
        self.__degrees_celcius_per_meter = self.__convert_from_base(TemperatureGradientUnits.DegreeCelsiusPerMeter)
        return self.__degrees_celcius_per_meter

    
    @property
    def degrees_fahrenheit_per_foot(self) -> float:
        """
        
        """
        if self.__degrees_fahrenheit_per_foot != None:
            return self.__degrees_fahrenheit_per_foot
        self.__degrees_fahrenheit_per_foot = self.__convert_from_base(TemperatureGradientUnits.DegreeFahrenheitPerFoot)
        return self.__degrees_fahrenheit_per_foot

    
    @property
    def degrees_celcius_per_kilometer(self) -> float:
        """
        
        """
        if self.__degrees_celcius_per_kilometer != None:
            return self.__degrees_celcius_per_kilometer
        self.__degrees_celcius_per_kilometer = self.__convert_from_base(TemperatureGradientUnits.DegreeCelsiusPerKilometer)
        return self.__degrees_celcius_per_kilometer

    
    def to_string(self, unit: TemperatureGradientUnits = TemperatureGradientUnits.KelvinPerMeter, fractional_digits: int = None) -> str:
        """
        Format the TemperatureGradient to a string.
        
        Note: the default format for TemperatureGradient is KelvinPerMeter.
        To specify the unit format, set the 'unit' parameter.
        
        Args:
            unit (str): The unit to format the TemperatureGradient. Default is 'KelvinPerMeter'.
            fractional_digits (int, optional): The number of fractional digits to keep.

        Returns:
            str: The string format of the Angle.
        """
        
        if unit == TemperatureGradientUnits.KelvinPerMeter:
            return f"""{super()._truncate_fraction_digits(self.kelvins_per_meter, fractional_digits)} ∆°K/m"""
        
        if unit == TemperatureGradientUnits.DegreeCelsiusPerMeter:
            return f"""{super()._truncate_fraction_digits(self.degrees_celcius_per_meter, fractional_digits)} ∆°C/m"""
        
        if unit == TemperatureGradientUnits.DegreeFahrenheitPerFoot:
            return f"""{super()._truncate_fraction_digits(self.degrees_fahrenheit_per_foot, fractional_digits)} ∆°F/ft"""
        
        if unit == TemperatureGradientUnits.DegreeCelsiusPerKilometer:
            return f"""{super()._truncate_fraction_digits(self.degrees_celcius_per_kilometer, fractional_digits)} ∆°C/km"""
        
        return f'{self._value}'


    def get_unit_abbreviation(self, unit_abbreviation: TemperatureGradientUnits = TemperatureGradientUnits.KelvinPerMeter) -> str:
        """
        Get TemperatureGradient unit abbreviation.
        Note! the default abbreviation for TemperatureGradient is KelvinPerMeter.
        To specify the unit abbreviation set the 'unit_abbreviation' parameter.
        """
        
        if unit_abbreviation == TemperatureGradientUnits.KelvinPerMeter:
            return """∆°K/m"""
        
        if unit_abbreviation == TemperatureGradientUnits.DegreeCelsiusPerMeter:
            return """∆°C/m"""
        
        if unit_abbreviation == TemperatureGradientUnits.DegreeFahrenheitPerFoot:
            return """∆°F/ft"""
        
        if unit_abbreviation == TemperatureGradientUnits.DegreeCelsiusPerKilometer:
            return """∆°C/km"""
        


from enum import Enum
import math

from ..abstract_unit import AbstractMeasure



class ForceUnits(Enum):
        """
            ForceUnits enumeration
        """
        
        Dyn = 'Dyn'
        """
            One dyne is equal to 10 micronewtons, 10e−5 N or to 10 nsn (nanosthenes) in the old metre–tonne–second system of units.
        """
        
        KilogramForce = 'KilogramForce'
        """
            The kilogram-force, or kilopond, is equal to the magnitude of the force exerted on one kilogram of mass in a 9.80665 m/s2 gravitational field (standard gravity). Therefore, one kilogram-force is by definition equal to 9.80665 N.
        """
        
        TonneForce = 'TonneForce'
        """
            The tonne-force, metric ton-force, megagram-force, and megapond (Mp) are each 1000 kilograms-force.
        """
        
        Newton = 'Newton'
        """
            The newton (symbol: N) is the unit of force in the International System of Units (SI). It is defined as 1 kg⋅m/s2, the force which gives a mass of 1 kilogram an acceleration of 1 metre per second per second.
        """
        
        KiloPond = 'KiloPond'
        """
            The kilogram-force, or kilopond, is equal to the magnitude of the force exerted on one kilogram of mass in a 9.80665 m/s2 gravitational field (standard gravity). Therefore, one kilogram-force is by definition equal to 9.80665 N.
        """
        
        Poundal = 'Poundal'
        """
            The poundal is defined as the force necessary to accelerate 1 pound-mass at 1 foot per second per second. 1 pdl = 0.138254954376 N exactly.
        """
        
        PoundForce = 'PoundForce'
        """
            The standard values of acceleration of the standard gravitational field (gn) and the international avoirdupois pound (lb) result in a pound-force equal to 4.4482216152605 N.
        """
        
        OunceForce = 'OunceForce'
        """
            An ounce-force is 1⁄16 of a pound-force, or about 0.2780139 newtons.
        """
        
        ShortTonForce = 'ShortTonForce'
        """
            The short ton-force is a unit of force equal to 2,000 pounds-force (907.18474 kgf), that is most commonly used in the United States – known there simply as the ton or US ton.
        """
        
        Micronewton = 'Micronewton'
        """
            
        """
        
        Millinewton = 'Millinewton'
        """
            
        """
        
        Decanewton = 'Decanewton'
        """
            
        """
        
        Kilonewton = 'Kilonewton'
        """
            
        """
        
        Meganewton = 'Meganewton'
        """
            
        """
        
        KilopoundForce = 'KilopoundForce'
        """
            
        """
        

class ForceDto:
    """
    A DTO representation of a Force

    Attributes:
        value (float): The value of the Force.
        unit (ForceUnits): The specific unit that the Force value is representing.
    """

    def __init__(self, value: float, unit: ForceUnits):
        """
        Create a new DTO representation of a Force

        Parameters:
            value (float): The value of the Force.
            unit (ForceUnits): The specific unit that the Force value is representing.
        """
        self.value: float = value
        """
        The value of the Force
        """
        self.unit: ForceUnits = unit
        """
        The specific unit that the Force value is representing
        """

    def to_json(self):
        """
        Get a Force DTO JSON object representing the current unit.

        :return: JSON object represents Force DTO.
        :rtype: dict
        :example return: {"value": 100, "unit": "Newton"}
        """
        return {"value": self.value, "unit": self.unit.value}

    @staticmethod
    def from_json(data):
        """
        Obtain a new instance of Force DTO from a json representation.

        :param data: The Force DTO in JSON representation.
        :type data: dict
        :example data: {"value": 100, "unit": "Newton"}
        :return: A new instance of ForceDto.
        :rtype: ForceDto
        """
        return ForceDto(value=data["value"], unit=ForceUnits(data["unit"]))


class Force(AbstractMeasure):
    """
    In physics, a force is any influence that causes an object to undergo a certain change, either concerning its movement, direction, or geometrical construction. In other words, a force can cause an object with mass to change its velocity (which includes to begin moving from a state of rest), i.e., to accelerate, or a flexible object to deform, or both. Force can also be described by intuitive concepts such as a push or a pull. A force has both magnitude and direction, making it a vector quantity. It is measured in the SI unit of newtons and represented by the symbol F.

    Args:
        value (float): The value.
        from_unit (ForceUnits): The Force unit to create from, The default unit is Newton
    """
    def __init__(self, value: float, from_unit: ForceUnits = ForceUnits.Newton):
        # Do not validate type, to allow working with numpay arrays and similar objects who supports all arithmetic 
        # operations, but they are not a number, see #14 
        # if math.isnan(value):
        #     raise ValueError('Invalid unit: value is NaN')
        self._value = self.__convert_to_base(value, from_unit)
        
        self.__dyne = None
        
        self.__kilograms_force = None
        
        self.__tonnes_force = None
        
        self.__newtons = None
        
        self.__kilo_ponds = None
        
        self.__poundals = None
        
        self.__pounds_force = None
        
        self.__ounce_force = None
        
        self.__short_tons_force = None
        
        self.__micronewtons = None
        
        self.__millinewtons = None
        
        self.__decanewtons = None
        
        self.__kilonewtons = None
        
        self.__meganewtons = None
        
        self.__kilopounds_force = None
        

    def convert(self, unit: ForceUnits) -> float:
        return self.__convert_from_base(unit)

    def to_dto(self, hold_in_unit: ForceUnits = ForceUnits.Newton) -> ForceDto:
        """
        Get a new instance of Force DTO representing the current unit.

        :param hold_in_unit: The specific Force unit to store the Force value in the DTO representation.
        :type hold_in_unit: ForceUnits
        :return: A new instance of ForceDto.
        :rtype: ForceDto
        """
        return ForceDto(value=self.convert(hold_in_unit), unit=hold_in_unit)
    
    def to_dto_json(self, hold_in_unit: ForceUnits = ForceUnits.Newton):
        """
        Get a Force DTO JSON object representing the current unit.

        :param hold_in_unit: The specific Force unit to store the Force value in the DTO representation.
        :type hold_in_unit: ForceUnits
        :return: JSON object represents Force DTO.
        :rtype: dict
        :example return: {"value": 100, "unit": "Newton"}
        """
        return self.to_dto(hold_in_unit).to_json()

    @staticmethod
    def from_dto(force_dto: ForceDto):
        """
        Obtain a new instance of Force from a DTO unit object.

        :param force_dto: The Force DTO representation.
        :type force_dto: ForceDto
        :return: A new instance of Force.
        :rtype: Force
        """
        return Force(force_dto.value, force_dto.unit)

    @staticmethod
    def from_dto_json(data: dict):
        """
        Obtain a new instance of Force from a DTO unit json representation.

        :param data: The Force DTO in JSON representation.
        :type data: dict
        :example data: {"value": 100, "unit": "Newton"}
        :return: A new instance of Force.
        :rtype: Force
        """
        return Force.from_dto(ForceDto.from_json(data))

    def __convert_from_base(self, from_unit: ForceUnits) -> float:
        value = self._value
        
        if from_unit == ForceUnits.Dyn:
            return (value * 1e5)
        
        if from_unit == ForceUnits.KilogramForce:
            return (value / 9.80665)
        
        if from_unit == ForceUnits.TonneForce:
            return (value / (9.80665 * 1000))
        
        if from_unit == ForceUnits.Newton:
            return (value)
        
        if from_unit == ForceUnits.KiloPond:
            return (value / 9.80665)
        
        if from_unit == ForceUnits.Poundal:
            return (value / 0.138254954376)
        
        if from_unit == ForceUnits.PoundForce:
            return (value / 4.4482216152605)
        
        if from_unit == ForceUnits.OunceForce:
            return (value / (4.4482216152605 / 16))
        
        if from_unit == ForceUnits.ShortTonForce:
            return (value / (4.4482216152605 * 2000))
        
        if from_unit == ForceUnits.Micronewton:
            return ((value) / 1e-06)
        
        if from_unit == ForceUnits.Millinewton:
            return ((value) / 0.001)
        
        if from_unit == ForceUnits.Decanewton:
            return ((value) / 10.0)
        
        if from_unit == ForceUnits.Kilonewton:
            return ((value) / 1000.0)
        
        if from_unit == ForceUnits.Meganewton:
            return ((value) / 1000000.0)
        
        if from_unit == ForceUnits.KilopoundForce:
            return ((value / 4.4482216152605) / 1000.0)
        
        return None


    def __convert_to_base(self, value: float, to_unit: ForceUnits) -> float:
        
        if to_unit == ForceUnits.Dyn:
            return (value / 1e5)
        
        if to_unit == ForceUnits.KilogramForce:
            return (value * 9.80665)
        
        if to_unit == ForceUnits.TonneForce:
            return (value * (9.80665 * 1000))
        
        if to_unit == ForceUnits.Newton:
            return (value)
        
        if to_unit == ForceUnits.KiloPond:
            return (value * 9.80665)
        
        if to_unit == ForceUnits.Poundal:
            return (value * 0.138254954376)
        
        if to_unit == ForceUnits.PoundForce:
            return (value * 4.4482216152605)
        
        if to_unit == ForceUnits.OunceForce:
            return (value * (4.4482216152605 / 16))
        
        if to_unit == ForceUnits.ShortTonForce:
            return (value * (4.4482216152605 * 2000))
        
        if to_unit == ForceUnits.Micronewton:
            return ((value) * 1e-06)
        
        if to_unit == ForceUnits.Millinewton:
            return ((value) * 0.001)
        
        if to_unit == ForceUnits.Decanewton:
            return ((value) * 10.0)
        
        if to_unit == ForceUnits.Kilonewton:
            return ((value) * 1000.0)
        
        if to_unit == ForceUnits.Meganewton:
            return ((value) * 1000000.0)
        
        if to_unit == ForceUnits.KilopoundForce:
            return ((value * 4.4482216152605) * 1000.0)
        
        return None


    @property
    def base_value(self) -> float:
        return self._value

    
    @staticmethod
    def from_dyne(dyne: float):
        """
        Create a new instance of Force from a value in dyne.

        One dyne is equal to 10 micronewtons, 10e−5 N or to 10 nsn (nanosthenes) in the old metre–tonne–second system of units.

        :param meters: The Force value in dyne.
        :type dyne: float
        :return: A new instance of Force.
        :rtype: Force
        """
        return Force(dyne, ForceUnits.Dyn)

    
    @staticmethod
    def from_kilograms_force(kilograms_force: float):
        """
        Create a new instance of Force from a value in kilograms_force.

        The kilogram-force, or kilopond, is equal to the magnitude of the force exerted on one kilogram of mass in a 9.80665 m/s2 gravitational field (standard gravity). Therefore, one kilogram-force is by definition equal to 9.80665 N.

        :param meters: The Force value in kilograms_force.
        :type kilograms_force: float
        :return: A new instance of Force.
        :rtype: Force
        """
        return Force(kilograms_force, ForceUnits.KilogramForce)

    
    @staticmethod
    def from_tonnes_force(tonnes_force: float):
        """
        Create a new instance of Force from a value in tonnes_force.

        The tonne-force, metric ton-force, megagram-force, and megapond (Mp) are each 1000 kilograms-force.

        :param meters: The Force value in tonnes_force.
        :type tonnes_force: float
        :return: A new instance of Force.
        :rtype: Force
        """
        return Force(tonnes_force, ForceUnits.TonneForce)

    
    @staticmethod
    def from_newtons(newtons: float):
        """
        Create a new instance of Force from a value in newtons.

        The newton (symbol: N) is the unit of force in the International System of Units (SI). It is defined as 1 kg⋅m/s2, the force which gives a mass of 1 kilogram an acceleration of 1 metre per second per second.

        :param meters: The Force value in newtons.
        :type newtons: float
        :return: A new instance of Force.
        :rtype: Force
        """
        return Force(newtons, ForceUnits.Newton)

    
    @staticmethod
    def from_kilo_ponds(kilo_ponds: float):
        """
        Create a new instance of Force from a value in kilo_ponds.

        The kilogram-force, or kilopond, is equal to the magnitude of the force exerted on one kilogram of mass in a 9.80665 m/s2 gravitational field (standard gravity). Therefore, one kilogram-force is by definition equal to 9.80665 N.

        :param meters: The Force value in kilo_ponds.
        :type kilo_ponds: float
        :return: A new instance of Force.
        :rtype: Force
        """
        return Force(kilo_ponds, ForceUnits.KiloPond)

    
    @staticmethod
    def from_poundals(poundals: float):
        """
        Create a new instance of Force from a value in poundals.

        The poundal is defined as the force necessary to accelerate 1 pound-mass at 1 foot per second per second. 1 pdl = 0.138254954376 N exactly.

        :param meters: The Force value in poundals.
        :type poundals: float
        :return: A new instance of Force.
        :rtype: Force
        """
        return Force(poundals, ForceUnits.Poundal)

    
    @staticmethod
    def from_pounds_force(pounds_force: float):
        """
        Create a new instance of Force from a value in pounds_force.

        The standard values of acceleration of the standard gravitational field (gn) and the international avoirdupois pound (lb) result in a pound-force equal to 4.4482216152605 N.

        :param meters: The Force value in pounds_force.
        :type pounds_force: float
        :return: A new instance of Force.
        :rtype: Force
        """
        return Force(pounds_force, ForceUnits.PoundForce)

    
    @staticmethod
    def from_ounce_force(ounce_force: float):
        """
        Create a new instance of Force from a value in ounce_force.

        An ounce-force is 1⁄16 of a pound-force, or about 0.2780139 newtons.

        :param meters: The Force value in ounce_force.
        :type ounce_force: float
        :return: A new instance of Force.
        :rtype: Force
        """
        return Force(ounce_force, ForceUnits.OunceForce)

    
    @staticmethod
    def from_short_tons_force(short_tons_force: float):
        """
        Create a new instance of Force from a value in short_tons_force.

        The short ton-force is a unit of force equal to 2,000 pounds-force (907.18474 kgf), that is most commonly used in the United States – known there simply as the ton or US ton.

        :param meters: The Force value in short_tons_force.
        :type short_tons_force: float
        :return: A new instance of Force.
        :rtype: Force
        """
        return Force(short_tons_force, ForceUnits.ShortTonForce)

    
    @staticmethod
    def from_micronewtons(micronewtons: float):
        """
        Create a new instance of Force from a value in micronewtons.

        

        :param meters: The Force value in micronewtons.
        :type micronewtons: float
        :return: A new instance of Force.
        :rtype: Force
        """
        return Force(micronewtons, ForceUnits.Micronewton)

    
    @staticmethod
    def from_millinewtons(millinewtons: float):
        """
        Create a new instance of Force from a value in millinewtons.

        

        :param meters: The Force value in millinewtons.
        :type millinewtons: float
        :return: A new instance of Force.
        :rtype: Force
        """
        return Force(millinewtons, ForceUnits.Millinewton)

    
    @staticmethod
    def from_decanewtons(decanewtons: float):
        """
        Create a new instance of Force from a value in decanewtons.

        

        :param meters: The Force value in decanewtons.
        :type decanewtons: float
        :return: A new instance of Force.
        :rtype: Force
        """
        return Force(decanewtons, ForceUnits.Decanewton)

    
    @staticmethod
    def from_kilonewtons(kilonewtons: float):
        """
        Create a new instance of Force from a value in kilonewtons.

        

        :param meters: The Force value in kilonewtons.
        :type kilonewtons: float
        :return: A new instance of Force.
        :rtype: Force
        """
        return Force(kilonewtons, ForceUnits.Kilonewton)

    
    @staticmethod
    def from_meganewtons(meganewtons: float):
        """
        Create a new instance of Force from a value in meganewtons.

        

        :param meters: The Force value in meganewtons.
        :type meganewtons: float
        :return: A new instance of Force.
        :rtype: Force
        """
        return Force(meganewtons, ForceUnits.Meganewton)

    
    @staticmethod
    def from_kilopounds_force(kilopounds_force: float):
        """
        Create a new instance of Force from a value in kilopounds_force.

        

        :param meters: The Force value in kilopounds_force.
        :type kilopounds_force: float
        :return: A new instance of Force.
        :rtype: Force
        """
        return Force(kilopounds_force, ForceUnits.KilopoundForce)

    
    @property
    def dyne(self) -> float:
        """
        One dyne is equal to 10 micronewtons, 10e−5 N or to 10 nsn (nanosthenes) in the old metre–tonne–second system of units.
        """
        if self.__dyne != None:
            return self.__dyne
        self.__dyne = self.__convert_from_base(ForceUnits.Dyn)
        return self.__dyne

    
    @property
    def kilograms_force(self) -> float:
        """
        The kilogram-force, or kilopond, is equal to the magnitude of the force exerted on one kilogram of mass in a 9.80665 m/s2 gravitational field (standard gravity). Therefore, one kilogram-force is by definition equal to 9.80665 N.
        """
        if self.__kilograms_force != None:
            return self.__kilograms_force
        self.__kilograms_force = self.__convert_from_base(ForceUnits.KilogramForce)
        return self.__kilograms_force

    
    @property
    def tonnes_force(self) -> float:
        """
        The tonne-force, metric ton-force, megagram-force, and megapond (Mp) are each 1000 kilograms-force.
        """
        if self.__tonnes_force != None:
            return self.__tonnes_force
        self.__tonnes_force = self.__convert_from_base(ForceUnits.TonneForce)
        return self.__tonnes_force

    
    @property
    def newtons(self) -> float:
        """
        The newton (symbol: N) is the unit of force in the International System of Units (SI). It is defined as 1 kg⋅m/s2, the force which gives a mass of 1 kilogram an acceleration of 1 metre per second per second.
        """
        if self.__newtons != None:
            return self.__newtons
        self.__newtons = self.__convert_from_base(ForceUnits.Newton)
        return self.__newtons

    
    @property
    def kilo_ponds(self) -> float:
        """
        The kilogram-force, or kilopond, is equal to the magnitude of the force exerted on one kilogram of mass in a 9.80665 m/s2 gravitational field (standard gravity). Therefore, one kilogram-force is by definition equal to 9.80665 N.
        """
        if self.__kilo_ponds != None:
            return self.__kilo_ponds
        self.__kilo_ponds = self.__convert_from_base(ForceUnits.KiloPond)
        return self.__kilo_ponds

    
    @property
    def poundals(self) -> float:
        """
        The poundal is defined as the force necessary to accelerate 1 pound-mass at 1 foot per second per second. 1 pdl = 0.138254954376 N exactly.
        """
        if self.__poundals != None:
            return self.__poundals
        self.__poundals = self.__convert_from_base(ForceUnits.Poundal)
        return self.__poundals

    
    @property
    def pounds_force(self) -> float:
        """
        The standard values of acceleration of the standard gravitational field (gn) and the international avoirdupois pound (lb) result in a pound-force equal to 4.4482216152605 N.
        """
        if self.__pounds_force != None:
            return self.__pounds_force
        self.__pounds_force = self.__convert_from_base(ForceUnits.PoundForce)
        return self.__pounds_force

    
    @property
    def ounce_force(self) -> float:
        """
        An ounce-force is 1⁄16 of a pound-force, or about 0.2780139 newtons.
        """
        if self.__ounce_force != None:
            return self.__ounce_force
        self.__ounce_force = self.__convert_from_base(ForceUnits.OunceForce)
        return self.__ounce_force

    
    @property
    def short_tons_force(self) -> float:
        """
        The short ton-force is a unit of force equal to 2,000 pounds-force (907.18474 kgf), that is most commonly used in the United States – known there simply as the ton or US ton.
        """
        if self.__short_tons_force != None:
            return self.__short_tons_force
        self.__short_tons_force = self.__convert_from_base(ForceUnits.ShortTonForce)
        return self.__short_tons_force

    
    @property
    def micronewtons(self) -> float:
        """
        
        """
        if self.__micronewtons != None:
            return self.__micronewtons
        self.__micronewtons = self.__convert_from_base(ForceUnits.Micronewton)
        return self.__micronewtons

    
    @property
    def millinewtons(self) -> float:
        """
        
        """
        if self.__millinewtons != None:
            return self.__millinewtons
        self.__millinewtons = self.__convert_from_base(ForceUnits.Millinewton)
        return self.__millinewtons

    
    @property
    def decanewtons(self) -> float:
        """
        
        """
        if self.__decanewtons != None:
            return self.__decanewtons
        self.__decanewtons = self.__convert_from_base(ForceUnits.Decanewton)
        return self.__decanewtons

    
    @property
    def kilonewtons(self) -> float:
        """
        
        """
        if self.__kilonewtons != None:
            return self.__kilonewtons
        self.__kilonewtons = self.__convert_from_base(ForceUnits.Kilonewton)
        return self.__kilonewtons

    
    @property
    def meganewtons(self) -> float:
        """
        
        """
        if self.__meganewtons != None:
            return self.__meganewtons
        self.__meganewtons = self.__convert_from_base(ForceUnits.Meganewton)
        return self.__meganewtons

    
    @property
    def kilopounds_force(self) -> float:
        """
        
        """
        if self.__kilopounds_force != None:
            return self.__kilopounds_force
        self.__kilopounds_force = self.__convert_from_base(ForceUnits.KilopoundForce)
        return self.__kilopounds_force

    
    def to_string(self, unit: ForceUnits = ForceUnits.Newton, fractional_digits: int = None) -> str:
        """
        Format the Force to a string.
        
        Note: the default format for Force is Newton.
        To specify the unit format, set the 'unit' parameter.
        
        Args:
            unit (str): The unit to format the Force. Default is 'Newton'.
            fractional_digits (int, optional): The number of fractional digits to keep.

        Returns:
            str: The string format of the Angle.
        """
        
        if unit == ForceUnits.Dyn:
            return f"""{super()._truncate_fraction_digits(self.dyne, fractional_digits)} dyn"""
        
        if unit == ForceUnits.KilogramForce:
            return f"""{super()._truncate_fraction_digits(self.kilograms_force, fractional_digits)} kgf"""
        
        if unit == ForceUnits.TonneForce:
            return f"""{super()._truncate_fraction_digits(self.tonnes_force, fractional_digits)} tf"""
        
        if unit == ForceUnits.Newton:
            return f"""{super()._truncate_fraction_digits(self.newtons, fractional_digits)} N"""
        
        if unit == ForceUnits.KiloPond:
            return f"""{super()._truncate_fraction_digits(self.kilo_ponds, fractional_digits)} kp"""
        
        if unit == ForceUnits.Poundal:
            return f"""{super()._truncate_fraction_digits(self.poundals, fractional_digits)} pdl"""
        
        if unit == ForceUnits.PoundForce:
            return f"""{super()._truncate_fraction_digits(self.pounds_force, fractional_digits)} lbf"""
        
        if unit == ForceUnits.OunceForce:
            return f"""{super()._truncate_fraction_digits(self.ounce_force, fractional_digits)} ozf"""
        
        if unit == ForceUnits.ShortTonForce:
            return f"""{super()._truncate_fraction_digits(self.short_tons_force, fractional_digits)} tf (short)"""
        
        if unit == ForceUnits.Micronewton:
            return f"""{super()._truncate_fraction_digits(self.micronewtons, fractional_digits)} μN"""
        
        if unit == ForceUnits.Millinewton:
            return f"""{super()._truncate_fraction_digits(self.millinewtons, fractional_digits)} mN"""
        
        if unit == ForceUnits.Decanewton:
            return f"""{super()._truncate_fraction_digits(self.decanewtons, fractional_digits)} daN"""
        
        if unit == ForceUnits.Kilonewton:
            return f"""{super()._truncate_fraction_digits(self.kilonewtons, fractional_digits)} kN"""
        
        if unit == ForceUnits.Meganewton:
            return f"""{super()._truncate_fraction_digits(self.meganewtons, fractional_digits)} MN"""
        
        if unit == ForceUnits.KilopoundForce:
            return f"""{super()._truncate_fraction_digits(self.kilopounds_force, fractional_digits)} klbf"""
        
        return f'{self._value}'


    def get_unit_abbreviation(self, unit_abbreviation: ForceUnits = ForceUnits.Newton) -> str:
        """
        Get Force unit abbreviation.
        Note! the default abbreviation for Force is Newton.
        To specify the unit abbreviation set the 'unit_abbreviation' parameter.
        """
        
        if unit_abbreviation == ForceUnits.Dyn:
            return """dyn"""
        
        if unit_abbreviation == ForceUnits.KilogramForce:
            return """kgf"""
        
        if unit_abbreviation == ForceUnits.TonneForce:
            return """tf"""
        
        if unit_abbreviation == ForceUnits.Newton:
            return """N"""
        
        if unit_abbreviation == ForceUnits.KiloPond:
            return """kp"""
        
        if unit_abbreviation == ForceUnits.Poundal:
            return """pdl"""
        
        if unit_abbreviation == ForceUnits.PoundForce:
            return """lbf"""
        
        if unit_abbreviation == ForceUnits.OunceForce:
            return """ozf"""
        
        if unit_abbreviation == ForceUnits.ShortTonForce:
            return """tf (short)"""
        
        if unit_abbreviation == ForceUnits.Micronewton:
            return """μN"""
        
        if unit_abbreviation == ForceUnits.Millinewton:
            return """mN"""
        
        if unit_abbreviation == ForceUnits.Decanewton:
            return """daN"""
        
        if unit_abbreviation == ForceUnits.Kilonewton:
            return """kN"""
        
        if unit_abbreviation == ForceUnits.Meganewton:
            return """MN"""
        
        if unit_abbreviation == ForceUnits.KilopoundForce:
            return """klbf"""
        


from enum import Enum
import math

from ..abstract_unit import AbstractMeasure



class DynamicViscosityUnits(Enum):
        """
            DynamicViscosityUnits enumeration
        """
        
        NewtonSecondPerMeterSquared = 'NewtonSecondPerMeterSquared'
        """
            
        """
        
        PascalSecond = 'PascalSecond'
        """
            
        """
        
        Poise = 'Poise'
        """
            
        """
        
        Reyn = 'Reyn'
        """
            
        """
        
        PoundForceSecondPerSquareInch = 'PoundForceSecondPerSquareInch'
        """
            
        """
        
        PoundForceSecondPerSquareFoot = 'PoundForceSecondPerSquareFoot'
        """
            
        """
        
        PoundPerFootSecond = 'PoundPerFootSecond'
        """
            
        """
        
        MillipascalSecond = 'MillipascalSecond'
        """
            
        """
        
        MicropascalSecond = 'MicropascalSecond'
        """
            
        """
        
        Centipoise = 'Centipoise'
        """
            
        """
        

class DynamicViscosityDto:
    """
    A DTO representation of a DynamicViscosity

    Attributes:
        value (float): The value of the DynamicViscosity.
        unit (DynamicViscosityUnits): The specific unit that the DynamicViscosity value is representing.
    """

    def __init__(self, value: float, unit: DynamicViscosityUnits):
        """
        Create a new DTO representation of a DynamicViscosity

        Parameters:
            value (float): The value of the DynamicViscosity.
            unit (DynamicViscosityUnits): The specific unit that the DynamicViscosity value is representing.
        """
        self.value: float = value
        """
        The value of the DynamicViscosity
        """
        self.unit: DynamicViscosityUnits = unit
        """
        The specific unit that the DynamicViscosity value is representing
        """

    def to_json(self):
        """
        Get a DynamicViscosity DTO JSON object representing the current unit.

        :return: JSON object represents DynamicViscosity DTO.
        :rtype: dict
        :example return: {"value": 100, "unit": "NewtonSecondPerMeterSquared"}
        """
        return {"value": self.value, "unit": self.unit.value}

    @staticmethod
    def from_json(data):
        """
        Obtain a new instance of DynamicViscosity DTO from a json representation.

        :param data: The DynamicViscosity DTO in JSON representation.
        :type data: dict
        :example data: {"value": 100, "unit": "NewtonSecondPerMeterSquared"}
        :return: A new instance of DynamicViscosityDto.
        :rtype: DynamicViscosityDto
        """
        return DynamicViscosityDto(value=data["value"], unit=DynamicViscosityUnits(data["unit"]))


class DynamicViscosity(AbstractMeasure):
    """
    The dynamic (shear) viscosity of a fluid expresses its resistance to shearing flows, where adjacent layers move parallel to each other with different speeds

    Args:
        value (float): The value.
        from_unit (DynamicViscosityUnits): The DynamicViscosity unit to create from, The default unit is NewtonSecondPerMeterSquared
    """
    def __init__(self, value: float, from_unit: DynamicViscosityUnits = DynamicViscosityUnits.NewtonSecondPerMeterSquared):
        # Do not validate type, to allow working with numpay arrays and similar objects who supports all arithmetic 
        # operations, but they are not a number, see #14 
        # if math.isnan(value):
        #     raise ValueError('Invalid unit: value is NaN')
        self._value = self.__convert_to_base(value, from_unit)
        
        self.__newton_seconds_per_meter_squared = None
        
        self.__pascal_seconds = None
        
        self.__poise = None
        
        self.__reyns = None
        
        self.__pounds_force_second_per_square_inch = None
        
        self.__pounds_force_second_per_square_foot = None
        
        self.__pounds_per_foot_second = None
        
        self.__millipascal_seconds = None
        
        self.__micropascal_seconds = None
        
        self.__centipoise = None
        

    def convert(self, unit: DynamicViscosityUnits) -> float:
        return self.__convert_from_base(unit)

    def to_dto(self, hold_in_unit: DynamicViscosityUnits = DynamicViscosityUnits.NewtonSecondPerMeterSquared) -> DynamicViscosityDto:
        """
        Get a new instance of DynamicViscosity DTO representing the current unit.

        :param hold_in_unit: The specific DynamicViscosity unit to store the DynamicViscosity value in the DTO representation.
        :type hold_in_unit: DynamicViscosityUnits
        :return: A new instance of DynamicViscosityDto.
        :rtype: DynamicViscosityDto
        """
        return DynamicViscosityDto(value=self.convert(hold_in_unit), unit=hold_in_unit)
    
    def to_dto_json(self, hold_in_unit: DynamicViscosityUnits = DynamicViscosityUnits.NewtonSecondPerMeterSquared):
        """
        Get a DynamicViscosity DTO JSON object representing the current unit.

        :param hold_in_unit: The specific DynamicViscosity unit to store the DynamicViscosity value in the DTO representation.
        :type hold_in_unit: DynamicViscosityUnits
        :return: JSON object represents DynamicViscosity DTO.
        :rtype: dict
        :example return: {"value": 100, "unit": "NewtonSecondPerMeterSquared"}
        """
        return self.to_dto(hold_in_unit).to_json()

    @staticmethod
    def from_dto(dynamic_viscosity_dto: DynamicViscosityDto):
        """
        Obtain a new instance of DynamicViscosity from a DTO unit object.

        :param dynamic_viscosity_dto: The DynamicViscosity DTO representation.
        :type dynamic_viscosity_dto: DynamicViscosityDto
        :return: A new instance of DynamicViscosity.
        :rtype: DynamicViscosity
        """
        return DynamicViscosity(dynamic_viscosity_dto.value, dynamic_viscosity_dto.unit)

    @staticmethod
    def from_dto_json(data: dict):
        """
        Obtain a new instance of DynamicViscosity from a DTO unit json representation.

        :param data: The DynamicViscosity DTO in JSON representation.
        :type data: dict
        :example data: {"value": 100, "unit": "NewtonSecondPerMeterSquared"}
        :return: A new instance of DynamicViscosity.
        :rtype: DynamicViscosity
        """
        return DynamicViscosity.from_dto(DynamicViscosityDto.from_json(data))

    def __convert_from_base(self, from_unit: DynamicViscosityUnits) -> float:
        value = self._value
        
        if from_unit == DynamicViscosityUnits.NewtonSecondPerMeterSquared:
            return (value)
        
        if from_unit == DynamicViscosityUnits.PascalSecond:
            return (value)
        
        if from_unit == DynamicViscosityUnits.Poise:
            return (value * 10)
        
        if from_unit == DynamicViscosityUnits.Reyn:
            return (value / 6.8947572931683613e3)
        
        if from_unit == DynamicViscosityUnits.PoundForceSecondPerSquareInch:
            return (value / 6.8947572931683613e3)
        
        if from_unit == DynamicViscosityUnits.PoundForceSecondPerSquareFoot:
            return (value / 4.7880258980335843e1)
        
        if from_unit == DynamicViscosityUnits.PoundPerFootSecond:
            return (value / 1.4881639)
        
        if from_unit == DynamicViscosityUnits.MillipascalSecond:
            return ((value) / 0.001)
        
        if from_unit == DynamicViscosityUnits.MicropascalSecond:
            return ((value) / 1e-06)
        
        if from_unit == DynamicViscosityUnits.Centipoise:
            return ((value * 10) / 0.01)
        
        return None


    def __convert_to_base(self, value: float, to_unit: DynamicViscosityUnits) -> float:
        
        if to_unit == DynamicViscosityUnits.NewtonSecondPerMeterSquared:
            return (value)
        
        if to_unit == DynamicViscosityUnits.PascalSecond:
            return (value)
        
        if to_unit == DynamicViscosityUnits.Poise:
            return (value / 10)
        
        if to_unit == DynamicViscosityUnits.Reyn:
            return (value * 6.8947572931683613e3)
        
        if to_unit == DynamicViscosityUnits.PoundForceSecondPerSquareInch:
            return (value * 6.8947572931683613e3)
        
        if to_unit == DynamicViscosityUnits.PoundForceSecondPerSquareFoot:
            return (value * 4.7880258980335843e1)
        
        if to_unit == DynamicViscosityUnits.PoundPerFootSecond:
            return (value * 1.4881639)
        
        if to_unit == DynamicViscosityUnits.MillipascalSecond:
            return ((value) * 0.001)
        
        if to_unit == DynamicViscosityUnits.MicropascalSecond:
            return ((value) * 1e-06)
        
        if to_unit == DynamicViscosityUnits.Centipoise:
            return ((value / 10) * 0.01)
        
        return None


    @property
    def base_value(self) -> float:
        return self._value

    
    @staticmethod
    def from_newton_seconds_per_meter_squared(newton_seconds_per_meter_squared: float):
        """
        Create a new instance of DynamicViscosity from a value in newton_seconds_per_meter_squared.

        

        :param meters: The DynamicViscosity value in newton_seconds_per_meter_squared.
        :type newton_seconds_per_meter_squared: float
        :return: A new instance of DynamicViscosity.
        :rtype: DynamicViscosity
        """
        return DynamicViscosity(newton_seconds_per_meter_squared, DynamicViscosityUnits.NewtonSecondPerMeterSquared)

    
    @staticmethod
    def from_pascal_seconds(pascal_seconds: float):
        """
        Create a new instance of DynamicViscosity from a value in pascal_seconds.

        

        :param meters: The DynamicViscosity value in pascal_seconds.
        :type pascal_seconds: float
        :return: A new instance of DynamicViscosity.
        :rtype: DynamicViscosity
        """
        return DynamicViscosity(pascal_seconds, DynamicViscosityUnits.PascalSecond)

    
    @staticmethod
    def from_poise(poise: float):
        """
        Create a new instance of DynamicViscosity from a value in poise.

        

        :param meters: The DynamicViscosity value in poise.
        :type poise: float
        :return: A new instance of DynamicViscosity.
        :rtype: DynamicViscosity
        """
        return DynamicViscosity(poise, DynamicViscosityUnits.Poise)

    
    @staticmethod
    def from_reyns(reyns: float):
        """
        Create a new instance of DynamicViscosity from a value in reyns.

        

        :param meters: The DynamicViscosity value in reyns.
        :type reyns: float
        :return: A new instance of DynamicViscosity.
        :rtype: DynamicViscosity
        """
        return DynamicViscosity(reyns, DynamicViscosityUnits.Reyn)

    
    @staticmethod
    def from_pounds_force_second_per_square_inch(pounds_force_second_per_square_inch: float):
        """
        Create a new instance of DynamicViscosity from a value in pounds_force_second_per_square_inch.

        

        :param meters: The DynamicViscosity value in pounds_force_second_per_square_inch.
        :type pounds_force_second_per_square_inch: float
        :return: A new instance of DynamicViscosity.
        :rtype: DynamicViscosity
        """
        return DynamicViscosity(pounds_force_second_per_square_inch, DynamicViscosityUnits.PoundForceSecondPerSquareInch)

    
    @staticmethod
    def from_pounds_force_second_per_square_foot(pounds_force_second_per_square_foot: float):
        """
        Create a new instance of DynamicViscosity from a value in pounds_force_second_per_square_foot.

        

        :param meters: The DynamicViscosity value in pounds_force_second_per_square_foot.
        :type pounds_force_second_per_square_foot: float
        :return: A new instance of DynamicViscosity.
        :rtype: DynamicViscosity
        """
        return DynamicViscosity(pounds_force_second_per_square_foot, DynamicViscosityUnits.PoundForceSecondPerSquareFoot)

    
    @staticmethod
    def from_pounds_per_foot_second(pounds_per_foot_second: float):
        """
        Create a new instance of DynamicViscosity from a value in pounds_per_foot_second.

        

        :param meters: The DynamicViscosity value in pounds_per_foot_second.
        :type pounds_per_foot_second: float
        :return: A new instance of DynamicViscosity.
        :rtype: DynamicViscosity
        """
        return DynamicViscosity(pounds_per_foot_second, DynamicViscosityUnits.PoundPerFootSecond)

    
    @staticmethod
    def from_millipascal_seconds(millipascal_seconds: float):
        """
        Create a new instance of DynamicViscosity from a value in millipascal_seconds.

        

        :param meters: The DynamicViscosity value in millipascal_seconds.
        :type millipascal_seconds: float
        :return: A new instance of DynamicViscosity.
        :rtype: DynamicViscosity
        """
        return DynamicViscosity(millipascal_seconds, DynamicViscosityUnits.MillipascalSecond)

    
    @staticmethod
    def from_micropascal_seconds(micropascal_seconds: float):
        """
        Create a new instance of DynamicViscosity from a value in micropascal_seconds.

        

        :param meters: The DynamicViscosity value in micropascal_seconds.
        :type micropascal_seconds: float
        :return: A new instance of DynamicViscosity.
        :rtype: DynamicViscosity
        """
        return DynamicViscosity(micropascal_seconds, DynamicViscosityUnits.MicropascalSecond)

    
    @staticmethod
    def from_centipoise(centipoise: float):
        """
        Create a new instance of DynamicViscosity from a value in centipoise.

        

        :param meters: The DynamicViscosity value in centipoise.
        :type centipoise: float
        :return: A new instance of DynamicViscosity.
        :rtype: DynamicViscosity
        """
        return DynamicViscosity(centipoise, DynamicViscosityUnits.Centipoise)

    
    @property
    def newton_seconds_per_meter_squared(self) -> float:
        """
        
        """
        if self.__newton_seconds_per_meter_squared != None:
            return self.__newton_seconds_per_meter_squared
        self.__newton_seconds_per_meter_squared = self.__convert_from_base(DynamicViscosityUnits.NewtonSecondPerMeterSquared)
        return self.__newton_seconds_per_meter_squared

    
    @property
    def pascal_seconds(self) -> float:
        """
        
        """
        if self.__pascal_seconds != None:
            return self.__pascal_seconds
        self.__pascal_seconds = self.__convert_from_base(DynamicViscosityUnits.PascalSecond)
        return self.__pascal_seconds

    
    @property
    def poise(self) -> float:
        """
        
        """
        if self.__poise != None:
            return self.__poise
        self.__poise = self.__convert_from_base(DynamicViscosityUnits.Poise)
        return self.__poise

    
    @property
    def reyns(self) -> float:
        """
        
        """
        if self.__reyns != None:
            return self.__reyns
        self.__reyns = self.__convert_from_base(DynamicViscosityUnits.Reyn)
        return self.__reyns

    
    @property
    def pounds_force_second_per_square_inch(self) -> float:
        """
        
        """
        if self.__pounds_force_second_per_square_inch != None:
            return self.__pounds_force_second_per_square_inch
        self.__pounds_force_second_per_square_inch = self.__convert_from_base(DynamicViscosityUnits.PoundForceSecondPerSquareInch)
        return self.__pounds_force_second_per_square_inch

    
    @property
    def pounds_force_second_per_square_foot(self) -> float:
        """
        
        """
        if self.__pounds_force_second_per_square_foot != None:
            return self.__pounds_force_second_per_square_foot
        self.__pounds_force_second_per_square_foot = self.__convert_from_base(DynamicViscosityUnits.PoundForceSecondPerSquareFoot)
        return self.__pounds_force_second_per_square_foot

    
    @property
    def pounds_per_foot_second(self) -> float:
        """
        
        """
        if self.__pounds_per_foot_second != None:
            return self.__pounds_per_foot_second
        self.__pounds_per_foot_second = self.__convert_from_base(DynamicViscosityUnits.PoundPerFootSecond)
        return self.__pounds_per_foot_second

    
    @property
    def millipascal_seconds(self) -> float:
        """
        
        """
        if self.__millipascal_seconds != None:
            return self.__millipascal_seconds
        self.__millipascal_seconds = self.__convert_from_base(DynamicViscosityUnits.MillipascalSecond)
        return self.__millipascal_seconds

    
    @property
    def micropascal_seconds(self) -> float:
        """
        
        """
        if self.__micropascal_seconds != None:
            return self.__micropascal_seconds
        self.__micropascal_seconds = self.__convert_from_base(DynamicViscosityUnits.MicropascalSecond)
        return self.__micropascal_seconds

    
    @property
    def centipoise(self) -> float:
        """
        
        """
        if self.__centipoise != None:
            return self.__centipoise
        self.__centipoise = self.__convert_from_base(DynamicViscosityUnits.Centipoise)
        return self.__centipoise

    
    def to_string(self, unit: DynamicViscosityUnits = DynamicViscosityUnits.NewtonSecondPerMeterSquared, fractional_digits: int = None) -> str:
        """
        Format the DynamicViscosity to a string.
        
        Note: the default format for DynamicViscosity is NewtonSecondPerMeterSquared.
        To specify the unit format, set the 'unit' parameter.
        
        Args:
            unit (str): The unit to format the DynamicViscosity. Default is 'NewtonSecondPerMeterSquared'.
            fractional_digits (int, optional): The number of fractional digits to keep.

        Returns:
            str: The string format of the Angle.
        """
        
        if unit == DynamicViscosityUnits.NewtonSecondPerMeterSquared:
            return f"""{super()._truncate_fraction_digits(self.newton_seconds_per_meter_squared, fractional_digits)} Ns/m²"""
        
        if unit == DynamicViscosityUnits.PascalSecond:
            return f"""{super()._truncate_fraction_digits(self.pascal_seconds, fractional_digits)} Pa·s"""
        
        if unit == DynamicViscosityUnits.Poise:
            return f"""{super()._truncate_fraction_digits(self.poise, fractional_digits)} P"""
        
        if unit == DynamicViscosityUnits.Reyn:
            return f"""{super()._truncate_fraction_digits(self.reyns, fractional_digits)} reyn"""
        
        if unit == DynamicViscosityUnits.PoundForceSecondPerSquareInch:
            return f"""{super()._truncate_fraction_digits(self.pounds_force_second_per_square_inch, fractional_digits)} lbf·s/in²"""
        
        if unit == DynamicViscosityUnits.PoundForceSecondPerSquareFoot:
            return f"""{super()._truncate_fraction_digits(self.pounds_force_second_per_square_foot, fractional_digits)} lbf·s/ft²"""
        
        if unit == DynamicViscosityUnits.PoundPerFootSecond:
            return f"""{super()._truncate_fraction_digits(self.pounds_per_foot_second, fractional_digits)} lb/ft·s"""
        
        if unit == DynamicViscosityUnits.MillipascalSecond:
            return f"""{super()._truncate_fraction_digits(self.millipascal_seconds, fractional_digits)} mPa·s"""
        
        if unit == DynamicViscosityUnits.MicropascalSecond:
            return f"""{super()._truncate_fraction_digits(self.micropascal_seconds, fractional_digits)} μPa·s"""
        
        if unit == DynamicViscosityUnits.Centipoise:
            return f"""{super()._truncate_fraction_digits(self.centipoise, fractional_digits)} cP"""
        
        return f'{self._value}'


    def get_unit_abbreviation(self, unit_abbreviation: DynamicViscosityUnits = DynamicViscosityUnits.NewtonSecondPerMeterSquared) -> str:
        """
        Get DynamicViscosity unit abbreviation.
        Note! the default abbreviation for DynamicViscosity is NewtonSecondPerMeterSquared.
        To specify the unit abbreviation set the 'unit_abbreviation' parameter.
        """
        
        if unit_abbreviation == DynamicViscosityUnits.NewtonSecondPerMeterSquared:
            return """Ns/m²"""
        
        if unit_abbreviation == DynamicViscosityUnits.PascalSecond:
            return """Pa·s"""
        
        if unit_abbreviation == DynamicViscosityUnits.Poise:
            return """P"""
        
        if unit_abbreviation == DynamicViscosityUnits.Reyn:
            return """reyn"""
        
        if unit_abbreviation == DynamicViscosityUnits.PoundForceSecondPerSquareInch:
            return """lbf·s/in²"""
        
        if unit_abbreviation == DynamicViscosityUnits.PoundForceSecondPerSquareFoot:
            return """lbf·s/ft²"""
        
        if unit_abbreviation == DynamicViscosityUnits.PoundPerFootSecond:
            return """lb/ft·s"""
        
        if unit_abbreviation == DynamicViscosityUnits.MillipascalSecond:
            return """mPa·s"""
        
        if unit_abbreviation == DynamicViscosityUnits.MicropascalSecond:
            return """μPa·s"""
        
        if unit_abbreviation == DynamicViscosityUnits.Centipoise:
            return """cP"""
        